﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using DEOCIOAPP.Models;
using System.Drawing;


// Formulario para insertar una nueva oferta en el portal
namespace DEOCIOAPP.Forms
{
    public partial class AddOffers : Form
    {
        public AddOffers()
        {
            InitializeComponent();

            label21.Parent = pictureBox1;
            label21.BackColor = Color.Transparent;

            //Rellenamos los COMBOS de Províncias y Categorías
            AddProvinces();
            AddComboCategories();

            // Determinamos las fechas válidas para insertar una oferta
            txtFechaInicio.Value = DateTime.Now;
            txtFechaInicio.MinDate = DateTime.Now;
            txtFechaFin.Value = DateTime.Now.AddMonths(1);
            txtFechaFin.MinDate = txtFechaInicio.Value.AddDays(1);

        }


        //**************************************************************************
        //Nombre: button1_Click()
        //Descripción: Se validan los datos del formulario y se inserta la oferta en la BBDD
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            //Definición de variables
            bool status = false;
            bool ImageExist = false;
            bool BestOfProduct = false;

            DateTime desde = DateTime.Now;
            DateTime hasta = DateTime.Now;
            DEOCIO_WS.ProductModel P = new DEOCIO_WS.ProductModel();
            List<DEOCIO_WS.ProductPhotoAdd> PhotoGrid = new List<DEOCIO_WS.ProductPhotoAdd>();

            if ((txtNombre.Text.Trim() != "") && (txtOferta.Text.Trim() != "") && (txtPrecio.Text.Trim() != "")
                && (txtDescripcion.Text.Trim() != "") && (txtDescripcionBreve.Text.Trim() != "") && (txtNombre.Text.Trim() != "")
                && (txtNumProducto.Text.Trim() != "") && (txtFechaInicio.Text.Trim() != "") && (txtFechaFin.Text.Trim() != "")
                && (txtDireccionL1.Text.Trim() != "") && (txtCP.Text.Trim() != "") && (txtCiudad.Text.Trim() != ""))
            {

                if (ValidaPrimaria() == false)
                {
                    MessageBox.Show("Debe marcar almenos una imágen como primaria", "Atención!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
                //Rellenamos el objeto con los datos del formulario
                P.Name = txtNombre.Text;
                P.OfferPrice = Convert.ToDecimal(txtOferta.Text);
                P.Price = Convert.ToDecimal(txtPrecio.Text);
                P.ProductDescription = txtDescripcion.Text;
                P.ProductShortDesc = txtDescripcionBreve.Text;
                P.ProductNumber = txtNumProducto.Text;
                DateTime.TryParse(txtFechaInicio.Text, out desde);
                P.SellStartDate = desde;
                DateTime.TryParse(txtFechaFin.Text, out hasta);
                P.SellEndDate = hasta;
                
                int cmbCat = 0;
                if ((cmbCategories.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbCat = Convert.ToInt32((cmbCategories.SelectedItem as ComboboxItem).Value);
                if (cmbCat.Equals(0))
                {
                    MessageBox.Show("Debe seleccionar una categoría", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
                else
                {
                    P.ProductSubcategoryID = cmbCat;
                }

                // Verificamos el número máximo de imagenes por Oferta
                if (grdImagenes.RowCount > 6)
                {
                    MessageBox.Show("No puede insertar más de 6 imágines");
                    return;
                }

                foreach (DataGridViewRow row in grdImagenes.Rows)
                {
                    if ((row.Cells[2].Size.IsEmpty) || (row.Cells[2].Value == null) || (row.Cells[3].Size.IsEmpty) || (row.Cells[3].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        // Leemos los datos del grid grdImagenes
                        string PathImagen = row.Cells[2].Value.ToString();
                        string PathThumbNail = row.Cells[3].Value.ToString();

                        // Añadimos los datos dentro del objeto ProductPhotoAdd
                        DEOCIO_WS.ProductPhotoAdd PPH = new DEOCIO_WS.ProductPhotoAdd();

                        PPH.LargePhotoFileName = Path.GetFileName(PathImagen);
                        PPH.LargePhotoType = "image/" + Path.GetExtension(PathImagen).Replace(".", "");

                        // Convertimos la imagen en un array binario para guardarla en la base de datos
                        PPH.LargeImage64Bits = Utils.ImageToBinary(PathThumbNail);

                        PPH.ThumbnailPhotoFileName = Path.GetFileName(PathThumbNail);
                        PPH.ThumbnailPhotoType = "image/" + Path.GetExtension(PathThumbNail).Replace(".", "");

                        // Convertimos la imagen en un array binario para guardarla en la base de datos
                        PPH.Thumbnail64Bits = Utils.ImageToBinary(PathImagen);

                        if (row.Cells[1].Value.ToString() == "P") PPH.primaria = "P";
                        else PPH.primaria = "";
                        PhotoGrid.Add(PPH);
                        ImageExist = true;

                    }
                }

                // Verificamos el número máximo de descripciones "lo mejor esta oferta"
                if (grdBestofProduct.RowCount > 6)
                {
                    MessageBox.Show("No puede insertar más de 6 descripciones 'Lo mejor de esta oferta...'");
                    return;
                }

                List<string> BP = new List<string>();
                foreach (DataGridViewRow row in grdBestofProduct.Rows)
                {
                    if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        string Bof = row.Cells[1].Value.ToString();
                        BP.Add(Bof);
                        BestOfProduct = true;
                    }
                }

                P.Location = new DEOCIO_WS.dtLocation();
                P.Location.AddressLine1 = txtDireccionL1.Text.Trim();
                P.Location.AddressLine2 = txtDireccionL2.Text.Trim();
                P.Location.City = txtCiudad.Text.Trim();
                P.Location.PostalCode = txtCP.Text.Trim();

                int cmbProv = 0;
                if ((cmbProvincias.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbProv = Convert.ToInt32((cmbProvincias.SelectedItem as ComboboxItem).Value);
                if (cmbProv.Equals(0))
                {
                    MessageBox.Show("Debe seleccionar una província", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
                else
                {
                    P.Location.StateProvinceID = cmbProv;
                }

                // Si no se ha insertado ninguna imagen, mostramos mensaje y abortamos.
                if (ImageExist == false)
                {
                    MessageBox.Show("Debe insertar almenos una imagen");
                    return;
                }
                // Si no se ha introducido ninguna descripción, mostramos mensaje y abortamos.
                if (BestOfProduct == false)
                {
                    MessageBox.Show("Debe insertar almenos una descripción de 'lo mejor de esta oferta'");
                    return;
                }

                P.ProductsPhotos = PhotoGrid.ToArray();
                P.BestOfDescription = BP.ToArray();

                // Hacemos la llamada al WebService para insertar la oferta
                Models.Products DataContextProduct = new Models.Products();
                status = DataContextProduct.AddProduct(P);

                //Analizamos el resultcode para saber el resultado de la llamada al WebService
                if (status == true) MessageBox.Show("Producto insertado correctamente...");
                else MessageBox.Show("Se ha producido un error durante la opearción. Vuelva a intentarlo pasados unos minutos");
            }
            else
            {
                MessageBox.Show("Debe rellenar todos los datos del formulario.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }


        //**************************************************************************
        //Nombre: button2_Click()
        //Descripción: Muestra el dialogo para seleccionar la imagen LargeImage
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            // Mostramos el dialogo para seleccionar la imagen.
            OpenFileDialog FileDialog = new OpenFileDialog();
            if (FileDialog.ShowDialog().Equals(DialogResult.OK))
            {
                txtImagen.Text = FileDialog.FileName;
            }

        }


        //**************************************************************************
        //Nombre: btnThumbNailLocator_Click()
        //Descripción: Muestra el dialogo para seleccionar la imagen ThumbNail
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void btnThumbNailLocator_Click(object sender, EventArgs e)
        {
            // Mostramos el dialogo para seleccionar la imagen.
            OpenFileDialog FileDialog = new OpenFileDialog();
            if (FileDialog.ShowDialog().Equals(DialogResult.OK))
            {
                txtThumbNail.Text = FileDialog.FileName;
            }
        }

        //**************************************************************************
        //Nombre: txtImagen_TextChanged()
        //Descripción: Al modificar el valor del campo txtImagen habilita\Deshabilita los botones
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void txtImagen_TextChanged_1(object sender, EventArgs e)
        {
            checkImageButtons();
        }

        
        //**************************************************************************
        //Nombre: button2_Click_1()
        //Descripción: Analiza si se han seleccionado correctamente las imágines
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void checkImageButtons()
        {
            if ((txtImagen.Text.Length > 0) && (txtThumbNail.Text.Length > 0))
            {
                btnAdd.Enabled = true;
            }
            else
            {
                btnAdd.Enabled = false;
            }
        }

        private void txtThumbNail_TextChanged_1(object sender, EventArgs e)
        {
            checkImageButtons();
        }


        //**************************************************************************
        //Nombre: button2_Click_1()
        //Descripción: Cierra el formulario.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button2_Click_1(object sender, EventArgs e)
        {
            // Pide confirmación al usuario.
             DialogResult status = MessageBox.Show("Está seguro que desea cerrar este formulario?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
             if (status.ToString() == "Yes")
             {
                 this.Close();
             }
        }

        //**************************************************************************
        //Nombre: btnAdd_Click()
        //Descripción: Añade las imagenes seleccionadas al DataGrid de imágenes.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnAdd_Click(object sender, EventArgs e)
        {
            try
            {
                string primaria = "";

                if (chkPrimaria.Checked)
                {
                    primaria = "P";
                    foreach (DataGridViewRow row in grdImagenes.Rows)
                    {
                        foreach (DataGridViewCell cell in row.Cells)
                        {
                            if ((cell.Size.IsEmpty) || (cell.Value == null))
                            {
                                continue;
                            }

                            if (cell.Value.ToString() == "P")
                            {
                                cell.Value = "";
                            }
                        }
                    }
                }
                else primaria = "";

                string[] content = new string[] { "false", primaria, txtImagen.Text, txtThumbNail.Text };
                grdImagenes.Rows.Add(content);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                txtImagen.Text = "";
                txtThumbNail.Text = "";
                chkPrimaria.Checked = false;
                checkImageButtons();
            }
        }

        //**************************************************************************
        //Nombre: button3_Click()
        //Descripción: Añade la descripción al DataGrid de descripciones.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button3_Click(object sender, EventArgs e)
        {
            string[] content = new string[] {"false", txtDescripcionBestOfProduct.Text };
            grdBestofProduct.Rows.Add(content);
            txtDescripcionBestOfProduct.Text = "";
            txtDescripcionBestOfProduct.Focus();
        }

        private void btnEliminar_Click(object sender, EventArgs e)
        {
            // Pide confirmación al usuario
            DialogResult status = MessageBox.Show("Desea eliminar las imagenes seleccionadas", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (status.ToString() == "Yes")
            {
                List<int> RtoRemove = new List<int>();

                foreach (DataGridViewRow row in grdImagenes.Rows)
                {
                    if ((row.Cells[2].Size.IsEmpty) || (row.Cells[2].Value == null) || (row.Cells[3].Size.IsEmpty) || (row.Cells[3].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        if (row.Cells[0].Value.Equals(true))
                        {
                            RtoRemove.Add(row.Index);
                        }
                    }
                }

                RtoRemove.Reverse();
                foreach (int Row in RtoRemove)
                {
                    grdImagenes.Rows.RemoveAt(Row);
                }
            }
        }


        //**************************************************************************
        //Nombre: button4_Click()
        //Descripción: Elimina del DataGrid de Descripciones las filas seleccionadas por el usuario.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button4_Click(object sender, EventArgs e)
        {
            // Pide confirmación al usuario
            DialogResult status = MessageBox.Show("Desea eliminar las descripciones seleccionadas", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
           
            if (status.ToString() == "Yes")
            {
                List<int> RtoRemove = new List<int>();

                foreach (DataGridViewRow row in grdBestofProduct.Rows)
                {
                    if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        if (row.Cells[0].Value.Equals(true))
                        {
                            RtoRemove.Add(row.Index);
                        }
                    }
                }
                RtoRemove.Reverse();
                foreach (int Row in RtoRemove)
                {
                    grdBestofProduct.Rows.RemoveAt(Row);
                }
            }
        }


        //**************************************************************************
        //Nombre: txtDescripcionBestOfProduct_TextChanged()
        //Descripción: Habilita o deshabilita los botones para insertar una nueva descripción
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void txtDescripcionBestOfProduct_TextChanged_1(object sender, EventArgs e)
        {
            if (txtDescripcionBestOfProduct.Text.Trim() != "")
            {
                btnAddBoP.Enabled = true;
            }
            else
            {
                btnAddBoP.Enabled = false;
            }
        }


        //**************************************************************************
        //Nombre: txtPrecio_Leave()
        //Descripción: Al salir del campo txtPrecio se valida que el valor introducido sea un entero.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void txtPrecio_Leave(object sender, EventArgs e)
        {
            try
            {
                int Precio = Int32.Parse(txtPrecio.Text);
                if (Precio > 0) txtPrecio.Text = Precio.ToString();
                else txtPrecio.Text = "";
            }
            catch (Exception)
            {
                MessageBox.Show("El campo 'Precio Anterior', debe ser un número entero sin decimales", "Atención!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtPrecio.Text = "";
            }
        }

        //**************************************************************************
        //Nombre: txtOferta_Leave()
        //Descripción: Al salir del campo txtOferta se valida que el valor introducido sea un entero.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void txtOferta_Leave(object sender, EventArgs e)
        {
            try
            {
                int Precio = Int32.Parse(txtOferta.Text);
                if (Precio > 0) txtOferta.Text = Precio.ToString();
                else txtOferta.Text = "";
            }
            catch (Exception)
            {
                MessageBox.Show("El campo 'Precio en Oferta', debe ser un número entero sin decimales", "Atención!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtOferta.Text = "";
            }
        }

        private void txtFechaInicio_Leave(object sender, EventArgs e)
        {
            txtFechaFin.MinDate = txtFechaInicio.Value.AddDays(1);
        }


        //**************************************************************************
        //Nombre: ValidaPrimaria()
        //Descripción: Valida el DataGrid de imágenes para garantizar que almenos una se ha marcado como imagen primaria.
        //@params [None]
        //@return bool
        //**************************************************************************
        public bool ValidaPrimaria()
        {
            bool primaria = false;
            // Para fila del DataGris de imágenes validamos si existe alguna marcada como primaria.
            foreach (DataGridViewRow row in grdImagenes.Rows)
            {
                if ((row.Cells[2].Size.IsEmpty) || (row.Cells[2].Value == null) || (row.Cells[3].Size.IsEmpty) || (row.Cells[3].Value == null))
                {
                    continue;
                }
                else
                {
                    if (row.Cells[1].Value.ToString() == "P") primaria = true;
                }
            }
            return primaria;
        }


        //**************************************************************************
        //Nombre: AddProvinces()
        //Descripción: Rellena los valores del COMBO de províncias
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddProvinces()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las províncias
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.msStateProvince[] SP = DataContextProduct.GetProvinces();
                cmbProvincias.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbProvincias.Items.Add(item);

                // Insertamos los valores en la COMBO
                foreach (DEOCIO_WS.msStateProvince mSP in SP)
                {
                    item = new ComboboxItem();
                    item.Text = mSP.Name;
                    item.Value = mSP.StateProvinceID;
                    cmbProvincias.Items.Add(item);
                }
                cmbProvincias.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddComboCategories()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();
                cmbCategories.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbCategories.Items.Add(item);

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    cmbCategories.Items.Add(item);
                }
                cmbCategories.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

    }
}
