﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DEOCIOAPP.Models;
using DEOCIOAPP.DEOCIO_WS;

// Formulario para añadir nuevos usuarios
namespace DEOCIOAPP.Forms
{
    public partial class AddUser : Form
    {
        public AddUser()
        {
            InitializeComponent();
            // Insertamos los valores del combo de Roles.

            label21.Parent = pictureBox1;
            label21.BackColor = Color.Transparent;

            addRolesToCmb();
            // Insertamos los valores del combo de Províncias.
            AddProvinces();
            // Insertamos los valores de la check list de Categorías.
            AddchkCategories();
            // Seleccionamos la opción por defecto del combo "Sufijo" a "Sr"
            cmbSufijo.SelectedIndex = 0;
        }


        //**************************************************************************
        //Nombre: addRolesToCmb()
        //Descripción: Rellena los valores del COMBO de Roles
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void addRolesToCmb()
        {
            try
            {
                // Realizamos la consulta al WebService para obtener todos los roles de usuarios de la BBDD
                Models.Customers DataContextCustomers = new Models.Customers();
                DEOCIO_WS.msRole[] Roles = DataContextCustomers.GetRoles();
                cmbRoles.Items.Clear();

                // Añadimos el primer registro en blanco
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbRoles.Items.Add(item);

                // Por cada Rol devuelto insertamos su valor en el COMO de Roles.
                foreach (msRole Rol in Roles)
                {
                    item = new ComboboxItem();
                    item.Text = Rol.Nombre;
                    item.Value = Rol.IdRole;
                    cmbRoles.Items.Add(item);
                }
                cmbRoles.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error, mostramos mensaje.
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddProvinces()
        //Descripción: Rellena los valores del COMBO de províncias
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddProvinces()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las províncias
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.msStateProvince[] SP = DataContextProduct.GetProvinces();
                cmbProvincias.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbProvincias.Items.Add(item);

                // Insertamos los valores en la COMBO
                foreach (DEOCIO_WS.msStateProvince mSP in SP)
                {
                    item = new ComboboxItem();
                    item.Text = mSP.Name;
                    item.Value = mSP.StateProvinceID;
                    cmbProvincias.Items.Add(item);
                }
                cmbProvincias.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje.
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: btnCancelar_Click()
        //Descripción: Cierra el formulario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //**************************************************************************
        //Nombre: btnCrear_Click()
        //Descripción: Crea un nuego usuario en la BBDD con los datos del formulario.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnCrear_Click(object sender, EventArgs e)
        {
            try
            {
                if ((txtNombre.Text.Trim() != "") && (txtPrimerApellido.Text.Trim() != "") && (txtSegundoApellido.Text.Trim() != "")
                    && (txtDNI.Text.Trim() != "") && (txtEmail.Text.Trim() != "") && (txtNombre.Text.Trim() != "")
                    && (txtPassword.Text.Trim() != "") && (txtDirLineOne.Text.Trim() != "") && (txtCp.Text.Trim() != "")
                    && (txtCity.Text.Trim() != "") && (TxtTelf.Text.Trim() != "") && (txtMovil.Text.Trim() != ""))
                {
                    // Validamos que las dos claves introducidas sean iguales.
                    if (txtPassword.Text.Trim() != txtConfirmPassword.Text.Trim())
                    {
                        // Si no lo son, mostramos mensaje y abortamos.
                        MessageBox.Show("Las contraseñas introducidas no son iguales", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    // Cogemos el valor del COMBO de províncias, y validamos que no esté en blanco
                    int cmbProv = 0;
                    string ProvinceId = "";
                    if ((cmbProvincias.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbProv = Convert.ToInt32((cmbProvincias.SelectedItem as ComboboxItem).Value);
                    if (cmbProv.Equals(0))
                    {
                        // Si no se ha seleccionado ningún valor, mostramos mensaje y aboratmos.
                        MessageBox.Show("Debe seleccionar una província", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    else
                    {
                        ProvinceId = (cmbProvincias.SelectedItem as ComboboxItem).Value.ToString();
                    }

                    // Cogemos el valor del COMBO de Roles, y validamos que no esté en blanco
                    int cmbRole = 0;
                    int Role = 0;
                    if ((cmbRoles.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbRole = Convert.ToInt32((cmbRoles.SelectedItem as ComboboxItem).Value);
                    if (cmbRole.Equals(0))
                    {
                        // Si no se ha seleccionado ningún valor, mostramos mensaje y aboratmos
                        MessageBox.Show("Debe seleccionar un Rol para este usuario", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    else
                    {
                        Role = Convert.ToInt32((cmbRoles.SelectedItem as ComboboxItem).Value);
                    }

                    // Cogemos el valor del Check List de Categorías, y validamos que no esté en blanco
                    List<int> Categorias = new List<int>();
                    foreach (object itemChecked in chkCategories.CheckedItems)
                    {
                        Categorias.Add(Convert.ToInt32((itemChecked as ComboboxItem).Value));
                    }

                    if (Categorias.Count.Equals(0))
                    {
                        // Si no se ha seleccionado ningún valor, mostramos mensaje y aboratmos
                        MessageBox.Show("Debe seleccionar al menos una categoría", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    // Hacemos la llamada al WebService con los datos recogidos del formulario.
                    Models.Customers DataContextCustomers = new Models.Customers();
                    string status = DataContextCustomers.AddUser(cmbSufijo.Text, txtNombre.Text, txtPrimerApellido.Text,
                                                                 txtSegundoApellido.Text, txtDNI.Text, txtEmail.Text, txtPassword.Text,
                                                                 txtDirLineOne.Text, txtCp.Text, txtCity.Text, ProvinceId, TxtTelf.Text,
                                                                 txtMovil.Text, true, true, Role, Categorias.ToArray());

                    // Mostramos un mensaje con el resultado de la opeación.
                    if (status == "1")
                    {
                        MessageBox.Show("Usuario insertado correctamente", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);

                        // Si todo ha ido bien, cerramos el formulario
                        this.Close();
                    }
                    else MessageBox.Show(status, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    // Si no se han rellenado todos los campos del formulario mostramos mensaje y aboratmos.
                    MessageBox.Show("Debe rellenar todos los datos del formulario.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error durante la llamada al WebService mostramos mensaje y abortamos.
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddchkCategories()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();

                chkCategories.Items.Clear();

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    ComboboxItem item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    chkCategories.Items.Add(item);
                }
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
