﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DEOCIOAPP.DEOCIO_WS;
using DEOCIOAPP.Models;

// Formulario para listar las ofertas publicadas.
namespace DEOCIOAPP.Forms
{
    public partial class ListOffers : Form
    {
        public ListOffers()
        {
            InitializeComponent();

            label10.Parent = pictureBox1;
            label10.BackColor = Color.Transparent;

            // Cargamos los valores del Grid de Ofertas    
            FillGrid();

            // Cargamos los valores del COMBO de categorías
            AddComboCategories();

            // Cargamos los valores del COMBO de categorías2
            AddComboCategories2();

            //Cargamos los valores del COMBO de províncias
            AddProvinces();
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddComboCategories()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();
                cmbCategories.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbCategories.Items.Add(item);

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    cmbCategories.Items.Add(item);
                }
                cmbCategories.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddComboCategories2()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();
                cmbCategories2.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbCategories2.Items.Add(item);

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    cmbCategories2.Items.Add(item);
                }
                cmbCategories2.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: FillGrid()
        //Descripción: Rellena el DataGird con los valores devueltos del WebService
        //@params [None]
        //@return [None]
        //**************************************************************************
        public void FillGrid()
        {
            try
            {
                // Limpiamos todas las filas del Grid de ofertas
                grdOffers.Rows.Clear();

                // Analizamos el formulario de "Consulta"
                string ProdName = null;
                string ProdNumber = null;
                string ProdDesc = null;
                int? Category = null;

                if (txtProdName.Text.Trim() != "") ProdName = txtProdName.Text.Trim();
                if (txtProdNum.Text.Trim() != "") ProdNumber = txtProdNum.Text.Trim();
                if (txtDescShort.Text.Trim() != "") ProdDesc = txtDescShort.Text.Trim();

                if (cmbCategories.SelectedIndex != -1)
                {
                    if ((cmbCategories.SelectedItem as ComboboxItem).Text != "")
                    {
                        Category = Convert.ToInt32((cmbCategories.SelectedItem as ComboboxItem).Value);
                    }
                }

                //Hacemos la llamada al WebService para cargar las ofertas insertadas.
                Models.Products DataContextProduct = new Models.Products();
                ProductModel[] PM = DataContextProduct.GetProducts(ProdName, ProdNumber, ProdDesc, Category);

                // Por cada oferta devuelta cargamos los valores en el DataGrid
                foreach (ProductModel P in PM)
                {
                    string[] content = new string[] { "false", P.ProductID.ToString(), P.Name, P.ProductNumber, P.ProductShortDesc, P.SellStartDate.ToString(), P.SellEndDate.ToString() };
                    int Index = grdOffers.Rows.Add(content);


                    // Si la oferta está deshabilitada pintamos el fondo de la fila de color gris
                    if (P.Status.Equals(1))
                    {
                        grdOffers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGray;
                    }
                    // Si la oferta está habilitada, pintamos el fondo de la fila de color verde
                    else
                    {
                        grdOffers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGreen;
                    }
                }
            }
            catch
            {
                MessageBox.Show("Se ha producido un error al recuperar los datos del servidor. Vuelva a intentarlo pasados unos minutos", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        //**************************************************************************
        //Nombre: btnEliminar_Click()
        //Descripción: Elimina las ofertas seleccionadas por el usuario, siempre que 
        //             estas, no tengan compras realizadas.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnEliminar_Click(object sender, EventArgs e)
        {
            //Se pide confirmación al usuario
            DialogResult status = MessageBox.Show("Desea eliminar las ofertas seleccionadas", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (status.ToString() == "Yes")
            {
                // Por cada oferta seleccionada
                foreach (DataGridViewRow row in grdOffers.Rows)
                {
                    if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        if (row.Cells[0].Value.Equals(true))
                        {
                            //Se realiza la llamada al WebService para eliminar las ofertas.
                            Models.Products DataContextProduct = new Models.Products();
                            string Resstatus = DataContextProduct.DeleteOffer(Convert.ToInt32(row.Cells[1].Value));
                            MessageBox.Show(Resstatus, "Resultado", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                }
                FillGrid();
            }
        }

        //**************************************************************************
        //Nombre: button2_Click()
        //Descripción: Actualiza los valores de DataGrid de Ofertas.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            FillGrid();
        }

        //**************************************************************************
        //Nombre: button3_Click()
        //Descripción: Cierra el formulario actual.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button3_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //**************************************************************************
        //Nombre: button4_Click()
        //Descripción: Abre el formulario para Añadir una nueva oferta.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button4_Click(object sender, EventArgs e)
        {
            AddOffers NewForm = new AddOffers();
            NewForm.MdiParent = this.ParentForm;
            NewForm.Show();
            this.Close();
        }

        //**************************************************************************
        //Nombre: AmpliaInfo()
        //Descripción: Amplia la información de la oferta seleccionada en otra pestaña.
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AmpliaInfo()
        {
            try
            {
                int index = -1;
                int contador = 0;
                foreach (DataGridViewRow row in grdOffers.Rows)
                {
                    if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        if (row.Cells[0].Value.Equals(true))
                        {
                            contador++;
                            index = Convert.ToInt32(row.Cells[1].Value);
                        }
                    }
                }

                // Si solo se ha seleccionado una oferta procedemos a mostrar su información
                if (contador == 1)
                {
                    Models.Products DataContextProduct = new Models.Products();
                    ProductModel PM = DataContextProduct.GetProductOnlyByID(index);

                    txtNombre.Text = PM.Name;
                    txtNumProducto.Text = PM.ProductNumber;
                    txtDescripcionBreve.Text = PM.ProductShortDesc;
                    txtDescripcion.Text = PM.ProductDescription;
                    txtFechaInicio.Value = PM.SellStartDate;
                    txtFechaFin.Value = Convert.ToDateTime(PM.SellEndDate);
                    txtPrecio.Text = Math.Round(PM.Price,2).ToString();
                    txtOferta.Text = Math.Round(PM.OfferPrice,2).ToString();
                    ProductId.Text = PM.ProductID.ToString();

                    txtDireccionL1.Text = PM.Location.AddressLine1;
                    txtDireccionL2.Text = PM.Location.AddressLine2;
                    txtCP.Text = PM.Location.PostalCode.ToString();
                    txtCiudad.Text = PM.Location.City;

                    if (PM.Status.Equals(0)) CHKSTATUS.Checked = true;
                    else CHKSTATUS.Checked = false;

                    grdBestofProduct.Rows.Clear();
                    foreach (string BO in PM.BestOfDescription)
                    {
                        string[] content = new string[] { "false", BO };
                        grdBestofProduct.Rows.Add(content);
                    }

                    grdImagenes.Rows.Clear();
                    foreach (DEOCIO_WS.ProductPhotoAdd PH in PM.ProductsPhotos)
                    {
                        string[] content = new string[] { "false", PH.primaria, PH.LargePhotoFileName, PH.ThumbnailPhotoType };
                        grdImagenes.Rows.Add(content);
                    }

                    // Seleccionamos el valor del combo de províncias según la província recuperada
                    int count = 0;
                    foreach (ComboboxItem CI in cmbProvincias.Items)
                    {
                        if (CI.Value.ToString().Trim() == PM.Location.StateProvinceID.ToString()) cmbProvincias.SelectedIndex = count;
                        count++;
                    }

                    // Seleccionamos el valor del combo de categorias según la categoría recuperada
                    count = 0;
                    foreach (ComboboxItem CI in cmbCategories2.Items)
                    {
                        if (CI.Value.ToString().Trim() == PM.ProductCategory.ToString().Trim()) cmbCategories2.SelectedIndex = count;
                        count++;
                    }


                    tabControl1.SelectedIndex = 1;
                }
                else
                {
                    MessageBox.Show("Seleccione solo una oferta para ampliar su información", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        
        //**************************************************************************
        //Nombre: btnEditar_Click()
        //Descripción: Amplia la información de la oferta seleccionada en otra pestaña.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnEditar_Click(object sender, EventArgs e)
        {
            AmpliaInfo();
        }

        //**************************************************************************
        //Nombre: btnBuscar_Click()
        //Descripción: Actualiza el DataGrid de ofertas con los criterios de búsqueda introducidos
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnBuscar_Click(object sender, EventArgs e)
        {
            FillGrid();
        }

        //**************************************************************************
        //Nombre: AddProvinces()
        //Descripción: Rellena los valores del COMBO de províncias
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddProvinces()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las províncias
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.msStateProvince[] SP = DataContextProduct.GetProvinces();
                cmbProvincias.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbProvincias.Items.Add(item);

                // Insertamos los valores en la COMBO
                foreach (DEOCIO_WS.msStateProvince mSP in SP)
                {
                    item = new ComboboxItem();
                    item.Text = mSP.Name;
                    item.Value = mSP.StateProvinceID;
                    cmbProvincias.Items.Add(item);
                }
                cmbProvincias.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: button2_Click_1()
        //Descripción: Habilita una oferta seleccionada
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button2_Click_1(object sender, EventArgs e)
        {
            int index = -1;
            int contador = 0;
            foreach (DataGridViewRow row in grdOffers.Rows)
            {
                if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                {
                    continue;
                }
                else
                {
                    if (row.Cells[0].Value.Equals(true))
                    {
                        contador++;
                        index = Convert.ToInt32(row.Cells[1].Value);
                    }
                }
            }


            // Si solo se ha seleccionado una oferta procedemos a actualizar su estado.
            if (contador == 1)
            {
                 Models.Products DataContextProduct = new Models.Products();
                 string result = DataContextProduct.UpdateProductStatus(index, 0);
                 if (result == "ok")
                 {
                     MessageBox.Show("Oferta habilitada correctamente", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                     FillGrid();
                 }
                 else MessageBox.Show("Error al actualizar el estado de la oferta", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
             }
            else
            {
                MessageBox.Show("Seleccione solo una oferta para actualitzar su estado", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
        }

        //**************************************************************************
        //Nombre: button1_Click()
        //Descripción: Deshabilita una oferta seleccionada
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            int index = -1;
            int contador = 0;
            foreach (DataGridViewRow row in grdOffers.Rows)
            {
                if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                {
                    continue;
                }
                else
                {
                    if (row.Cells[0].Value.Equals(true))
                    {
                        contador++;
                        index = Convert.ToInt32(row.Cells[1].Value);
                    }
                }
            }


            // Si solo se ha seleccionado una oferta procedemos a actualizar su estado.
            if (contador == 1)
            {
                Models.Products DataContextProduct = new Models.Products();
                string result = DataContextProduct.UpdateProductStatus(index, 1);
                if (result == "ok")
                {
                    MessageBox.Show("Oferta deshabilitada correctamente", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    FillGrid();
                }
                else MessageBox.Show("Error al actualizar el estado de la oferta", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                MessageBox.Show("Seleccione solo una oferta para actualitzar su estado", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
        }

        private void button4_Click_1(object sender, EventArgs e)
        {
            EditMode(false);
        }


        //**************************************************************************
        //Nombre: EditMode()
        //Descripción: Habilita o Deshabilita el formulario de edición.
        //@params [bool Enable]
        //@return [None]
        //**************************************************************************
        private void EditMode(bool Enable)
        {
            txtNombre.Enabled = !Enable;
            txtNumProducto.Enabled = !Enable;
            txtDescripcionBreve.Enabled = !Enable;
            txtDescripcion.Enabled = !Enable;
            txtPrecio.Enabled = !Enable;
            txtOferta.Enabled = !Enable;
            txtFechaInicio.Enabled = !Enable;
            txtFechaFin.Enabled = !Enable;
            cmbCategories2.Enabled = !Enable;
            panel1.Enabled = Enable;
            btneditmode.Enabled = Enable;
            btnCancelar.Enabled = !Enable;
            btnCrear.Enabled = !Enable;
            CHKSTATUS.Enabled = !Enable;
        }

        private void btnCancelar_Click(object sender, EventArgs e)
        {
            EditMode(true);
            AmpliaInfo();
        }

        private void btnCrear_Click(object sender, EventArgs e)
        {
            DateTime desde = DateTime.Now;
            DateTime hasta = DateTime.Now;
            DEOCIO_WS.ProductModel P = new DEOCIO_WS.ProductModel();
            List<DEOCIO_WS.ProductPhotoAdd> PhotoGrid = new List<DEOCIO_WS.ProductPhotoAdd>();

            try
            {
                if ((txtNombre.Text.Trim() != "") && (txtOferta.Text.Trim() != "") && (txtPrecio.Text.Trim() != "")
               && (txtDescripcion.Text.Trim() != "") && (txtDescripcionBreve.Text.Trim() != "") && (txtNombre.Text.Trim() != "")
               && (txtNumProducto.Text.Trim() != "") && (txtFechaInicio.Text.Trim() != "") && (txtFechaFin.Text.Trim() != ""))
                {
                    //Rellenamos el objeto con los datos del formulario
                    P.Name = txtNombre.Text;
                    P.OfferPrice = Convert.ToDecimal(txtOferta.Text);
                    P.Price = Convert.ToDecimal(txtPrecio.Text);
                    P.ProductDescription = txtDescripcion.Text;
                    P.ProductShortDesc = txtDescripcionBreve.Text;
                    P.ProductNumber = txtNumProducto.Text;
                    DateTime.TryParse(txtFechaInicio.Text, out desde);
                    P.SellStartDate = desde;
                    DateTime.TryParse(txtFechaFin.Text, out hasta);
                    P.SellEndDate = hasta;
                    P.ProductID = Convert.ToInt32(ProductId.Text.Trim());

                    if (CHKSTATUS.Checked) P.Status = 0;
                    else P.Status = 1;

                    int cmbCat = 0;
                    if ((cmbCategories2.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbCat = Convert.ToInt32((cmbCategories2.SelectedItem as ComboboxItem).Value);
                    if (cmbCat.Equals(0))
                    {
                        MessageBox.Show("Debe seleccionar una categoría", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    else
                    {
                        P.ProductSubcategoryID = cmbCat;
                    }

                    Models.Products DataContextProduct = new Models.Products();
                    DataContextProduct.UpdateProducts(P);

                    EditMode(true);
                    AmpliaInfo();
                    MessageBox.Show("Datos Actualizados correctamente", "ATENCIÓN", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    CleanForm();
                    FillGrid();
                }
                else
                {
                    MessageBox.Show("Debe rellenar todos los datos del formulario.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CleanForm()
        {
            txtNombre.Text = "";
            txtNumProducto.Text ="";
            txtDescripcionBreve.Text ="";
            txtDescripcion.Text = "";
            txtFechaInicio.Value = DateTime.Now;
            txtFechaFin.Value = DateTime.Now;
            txtPrecio.Text = "";
            txtOferta.Text = "";
            ProductId.Text = "";
            CHKSTATUS.Checked = false;

            txtDireccionL1.Text = "";
            txtCP.Text = "";
            txtCiudad.Text = "";

            grdBestofProduct.Rows.Clear();
            grdImagenes.Rows.Clear();

            cmbProvincias.SelectedIndex = -1;
            cmbCategories2.SelectedIndex = -1;

            tabControl1.SelectedIndex = 0;
        }

        //**************************************************************************
        //Nombre: txtPrecio_Leave()
        //Descripción: Al salir del campo txtPrecio se valida que el valor introducido sea un entero.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void txtPrecio_Leave(object sender, EventArgs e)
        {
            try
            {
                int Precio = Int32.Parse(txtPrecio.Text);
                if (Precio > 0) txtPrecio.Text = Precio.ToString();
                else txtPrecio.Text = "";
            }
            catch (Exception)
            {
                MessageBox.Show("El campo 'Precio Anterior', debe ser un número entero sin decimales", "Atención!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtPrecio.Text = "";
            }
        }

        //**************************************************************************
        //Nombre: txtOferta_Leave()
        //Descripción: Al salir del campo txtOferta se valida que el valor introducido sea un entero.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void txtOferta_Leave(object sender, EventArgs e)
        {
            try
            {
                int Precio = Int32.Parse(txtOferta.Text);
                if (Precio > 0) txtOferta.Text = Precio.ToString();
                else txtOferta.Text = "";
            }
            catch (Exception)
            {
                MessageBox.Show("El campo 'Precio en Oferta', debe ser un número entero sin decimales", "Atención!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtOferta.Text = "";
            }
        }
    }
}
