﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DEOCIOAPP.DEOCIO_WS;
using DEOCIOAPP.Models;

namespace DEOCIOAPP.Forms
{
    public partial class ListOrders : Form
    {
        public ListOrders()
        {
            InitializeComponent();
            // Cargamos los valores de las órdenes de compra en el grid
            FillGridOrders(false);

            label7.Parent = pictureBox1;
            label7.BackColor = Color.Transparent;
            dateTimePicker1.Value = DateTime.Now;
            AddComboCategories();
            AddComboCategories2();

        }


        //**************************************************************************
        //Nombre: FillGridOrders()
        //Descripción: Rellena el DataGird con los valores devueltos del WebService
        //@params [None]
        //@return [None]
        //**************************************************************************
        public void FillGridOrders(bool Filter)
        {
            try
            {
                // Limpiamos todas las filas del Grid de pedidos
                grdOrdenes.Rows.Clear();

                //Hacemos la llamada al WebService para cargar los pedidos cargados
                Models.Products DataContextProduct = new Models.Products();
               
                int? SalesOrderID = null;
                DateTime? OrderDate = null;
                string ProductShortDesc = null;
                int? ProductSubcategoryID = null;
                string ProductNumber = null;
                string Email = null; 
                string DNI = null;

                if (Filter)
                {
                    if (txtProdNum.Text.Trim() != "")
                    {
                        try
                        {
                            SalesOrderID = Convert.ToInt32(txtProdNum.Text.Trim());
                        }
                        catch
                        {
                            MessageBox.Show("El campo Número debe ser un entero.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            SalesOrderID = null;
                        }
                    }

                    if (cmbCategories.SelectedIndex != -1)
                    {
                        if ((cmbCategories.SelectedItem as ComboboxItem).Text != "")
                        {
                            ProductSubcategoryID = Convert.ToInt32((cmbCategories.SelectedItem as ComboboxItem).Value);
                        }
                    }

                    if (txtDescShort.Text.Trim() != "") ProductShortDesc = txtDescShort.Text.Trim();
                    if (ProdNum.Text.Trim() != "") ProductNumber = ProdNum.Text.Trim();
                    if (txtEmailQry.Text.Trim() != "") Email = txtEmailQry.Text.Trim();
                    if (txtDniQry.Text.Trim() != "") DNI = txtDniQry.Text.Trim();
                    OrderDate = dateTimePicker1.Value;
                }

                CommandInfo[] PM = DataContextProduct.GetAllOrders(SalesOrderID, OrderDate, ProductShortDesc, ProductSubcategoryID, ProductNumber, Email, DNI);

                // Por cada oferta devuelta cargamos los valores en el DataGrid
                foreach (CommandInfo P in PM)
                {
                    string[] content = new string[] { "false", P.SalesOrderID.ToString(), P.OrderDate.ToString(), P.ProductID.ToString() , P.ProductShortDesc.ToString(), P.FirstName + " " + P.MiddleName + " " + P.LastName , P.DNI, P.Phone, P.Email, Math.Round(P.LineTotal,2).ToString() + "€" };
                    grdOrdenes.Rows.Add(content);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnBuscar_Click(object sender, EventArgs e)
        {
            FillGridOrders(true);
        }

        private void btnActualizarLista_Click(object sender, EventArgs e)
        {
            FillGridOrders(false);
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddComboCategories()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();
                cmbCategories.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbCategories.Items.Add(item);

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    cmbCategories.Items.Add(item);
                }
                cmbCategories.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddComboCategories2()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();
                cmbCategories2.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbCategories2.Items.Add(item);

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    cmbCategories2.Items.Add(item);
                }
                cmbCategories2.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnEditar_Click(object sender, EventArgs e)
        {
              try
            {
                int index = -1;
                int contador = 0;
                foreach (DataGridViewRow row in grdOrdenes.Rows)
                {
                    if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        if (row.Cells[0].Value.Equals(true))
                        {
                            contador++;
                            index = Convert.ToInt32(row.Cells[1].Value);
                        }
                    }
                }

                // Si solo se ha seleccionado una oferta procedemos a mostrar su información
                if (contador == 1)
                {
                    Models.Products DataContextProduct = new Models.Products();
                    CommandInfo[] PM = DataContextProduct.GetOrderById(index);

                    txtNombre1.Text = PM[0].FirstName;
                    txtPrimerApellido1.Text = PM[0].MiddleName;
                    txtSegundoApellido1.Text = PM[0].LastName;
                    txtDescripcion.Text = PM[0].ProductDescription;
                    txtDescripcionBreve.Text = PM[0].ProductShortDesc;
                    txtFechaInicio.Value = PM[0].SellStartDate;
                    txtFechaFin.Value = Convert.ToDateTime(PM[0].SellEndDate);
                    txtPrecio.Text = PM[0].Price.ToString();
                    txtOferta.Text = PM[0].OfferPrice.ToString();
                    txtNumProducto.Text = PM[0].ProductNumber.ToString();
                    txtNombre.Text = PM[0].Name;
                    txtDNI1.Text = PM[0].DNI;
                    TxtTelf.Text = PM[0].Phone;
                    txtMovil.Text = PM[0].Mobile;
                    txtPrecioUnitario.Text = PM[0].UnitPriceOffer.ToString();
                    txtUnidades.Text = PM[0].OrderQty.ToString();
                    txtPrecioTotal.Text = PM[0].LineTotal.ToString();
                    txtFechaCompra.Text = PM[0].OrderDate.ToString();
                    txtNumPedido.Text = PM[0].SalesOrderID.ToString();
                    tabControl1.SelectedIndex = 1;
                    txtEmail1.Text = PM[0].Email;


                    // Seleccionamos el valor del combo de categorias según la categoría recuperada
                    int count = 0;
                    foreach (ComboboxItem CI in cmbCategories2.Items)
                    {
                        if (CI.Value.ToString().Trim() == PM[0].Categories.ToString().Trim()) cmbCategories2.SelectedIndex = count;
                        count++;
                    }
                    
                    count = 0;
                    foreach (string CI in cmbSufijo.Items)
                    {
                        if (CI.Trim() == PM[0].Suffix.ToString().Trim()) cmbSufijo.SelectedIndex = count;
                        count++;
                    }
                }
                else
                {
                    MessageBox.Show("Seleccione solo un pedido para ampliar su información", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
            }
              catch (Exception ex)
              {
                  MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
              }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            txtProdNum.Text = "";
            cmbCategories.SelectedIndex = -1;
            txtDescShort.Text = "";
            ProdNum.Text = "";
            txtEmailQry.Text= "";
            txtDniQry.Text = "";
            dateTimePicker1.Value = DateTime.Now;
            FillGridOrders(false);
        }
    }
}
