﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DEOCIOAPP.DEOCIO_WS;
using DEOCIOAPP.Models;

namespace DEOCIOAPP.Forms
{
    public partial class ListUsers : Form
    {
        //**************************************************************************
        //Nombre: ListUsers()
        //Descripción: Inicializa el formulario
        //@params [None]
        //@return [None]
        //**************************************************************************
        public ListUsers()
        {
            InitializeComponent();
            // Rellena el DataGrid de clientes

            label10.Parent = pictureBox1;
            label10.BackColor = Color.Transparent;

            FillGridCustomers();
            // Rellena el Combo de Roles de usuario
            addRolesToCmb();

            addRolesToEditCmb();
            // Insertamos los valores del combo de Províncias.
            AddProvinces();
            // Insertamos los valores de la check list de Categorías.
            AddchkCategories();
            // Seleccionamos la opción por defecto del combo "Sufijo" a "Sr"
            cmbSufijo.SelectedIndex = 0;
        }

        //**************************************************************************
        //Nombre: FillGridCustomers()
        //Descripción: Rellena los datos del DataGrid de clientes en base al filtro introducido
        //@params [None]
        //@return [None]
        //**************************************************************************
        public void FillGridCustomers()
        {
            // Analizamos los valores del formulario "Filtrar Clientes"
            string Name = null;
            string MiddleName = null;
            string LastName = null;
            string DNI = null;
            string eMail = null;
            int? Role = null;
            int CmbRole = 0;

            if (txtNombre.Text.Trim() != "") Name = txtNombre.Text.Trim();
            if (txtPrimerApellido.Text.Trim() != "") MiddleName = txtPrimerApellido.Text.Trim();
            if (txtSegundoApellido.Text.Trim() != "") LastName = txtSegundoApellido.Text.Trim();
            if (txtDNI.Text.Trim() != "") DNI = txtDNI.Text.Trim();
            if (txtEmail.Text.Trim() != "") eMail = txtEmail.Text.Trim();

            if (cmbRoles.SelectedItem != null) CmbRole = Convert.ToInt32((cmbRoles.SelectedItem as ComboboxItem).Value);
            if (CmbRole > 0) Role = CmbRole;

            // Hacemos la llamada al WebService con los datos del filtro
            Models.Customers DataContextCustomers = new Models.Customers();
            dtCustomer_RecoveryAllResult[] CU = DataContextCustomers.GetAllCustomers(Name, MiddleName, LastName, DNI, eMail, Role);

            // Limpiamos las filas del DataGrid de Clientes
            grdCustomers.Rows.Clear();

            // Por cada valor devuelto por el WebService, insertamos los valores en el DataGrid de Clientes.
            foreach (dtCustomer_RecoveryAllResult C in CU)
            {
                if (C.EmailPromotion == 1)
                {
                    string[] content = new string[] { "false", C.CustomerID.ToString(), C.FirstName, C.MiddleName, C.LastName, C.Nombre, C.DNI, C.Email, C.Phone, C.Mobile };

                    DataGridViewCellStyle style = new DataGridViewCellStyle();

                    int Index = grdCustomers.Rows.Add(content);

                    // Si el usuario está bloqueado pintamos el fondo de la fila de color gris
                    if (C.IsLockedOut.Equals(1))
                    {
                        grdCustomers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGray;
                    }
                    // Si el usuario está habilitado, pintamos el fondo de la fila de color verde
                    else
                    {
                        grdCustomers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGreen;
                    }
                }
            }
        }

        //**************************************************************************
        //Nombre: button1_Click()
        //Descripción: Actualiza los datos del DataGrid de Usuarios con los valores del filtro
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            FillGridCustomers();
        }

        //**************************************************************************
        //Nombre: button2_Click()
        //Descripción: Cierra el formulario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //**************************************************************************
        //Nombre: addRolesToCmb()
        //Descripción: Inserta los valores de los Roles de la BBDD en el Combo de Roles
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void addRolesToCmb()
        {
            try
            {
                // Hacemos la llamada al Web Service
                Models.Customers DataContextCustomers = new Models.Customers();
                DEOCIO_WS.msRole[] Roles = DataContextCustomers.GetRoles();

                // Eliminamos todos los valores de la COMBO
                cmbRoles.Items.Clear();

                //Insertamos el primer valor en blanco
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "0";
                cmbRoles.Items.Add(item);

                // Por cada valor devuelto del WebService insertamos los valores en la COMBO
                foreach (msRole Rol in Roles)
                {
                    item = new ComboboxItem();
                    item.Text = Rol.Nombre;
                    item.Value = Rol.IdRole;
                    cmbRoles.Items.Add(item);
                }

                // Seleccionamos el valor en blanco.
                cmbRoles.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: btnEliminar_Click()
        //Descripción: Elimina los usuarios seleccionados de la BBDD
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnEliminar_Click(object sender, EventArgs e)
        {
            try
            {
                // Pedimos confirmación al usuario
                DialogResult status = MessageBox.Show("Desea eliminar los usuarios seleccionados?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (status.ToString() == "Yes")
                {
                    // Por cada registro seleccionado del datagrid de clientes
                    foreach (DataGridViewRow row in grdCustomers.Rows)
                    {
                        // Validamos que no se trate de una fila sin valor.
                        if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                        {
                            continue;
                        }
                        else
                        {
                            // Si la fila está seleccionada
                            if (row.Cells[0].Value.Equals(true))
                            {
                                // Hacemos la llamada al WebService para eliminar al usuario
                                Models.Customers DataContextCustomers = new Models.Customers();
                                string Resstatus = DataContextCustomers.DelCustomer(Convert.ToInt32(row.Cells[1].Value));

                                // Mostramos el mensaje de respuesta 
                                MessageBox.Show(Resstatus, "Resultado", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                    }
                    FillGridCustomers();
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: btnBuscar_Click()
        //Descripción: Actualizamos el DataGrid de Clientes con los datos del formulario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnBuscar_Click(object sender, EventArgs e)
        {
            FillGridCustomers();
        }

        //**************************************************************************
        //Nombre: button3_Click()
        //Descripción: Bloquea al usuario en la base de datos para que no pueda hacer login.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button3_Click(object sender, EventArgs e)
        {
            try
            {
                // Pedimos confirmación al usuario antes de proceder
                DialogResult status = MessageBox.Show("Desea bloquear la cuenta de los usuarios seleccionados?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (status.ToString() == "Yes")
                {
                    // Por cada fila del DataGrid de Clientes
                    foreach (DataGridViewRow row in grdCustomers.Rows)
                    {
                        // Comprovamos que la fila tenga valor
                        if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                        {
                            continue;
                        }
                        else
                        {
                            // Si la fila está seleccionada
                            if (row.Cells[0].Value.Equals(true))
                            {
                                // Hacemos la llamada al WebService para dar de baja al cliente
                                Models.Customers DataContextCustomers = new Models.Customers();
                                string Resstatus = DataContextCustomers.UpdateStatusAccount(Convert.ToInt32(row.Cells[1].Value), 1);

                                //Mostramos el resultado de la operación
                                MessageBox.Show(Resstatus, "Resultado", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                    }
                    // Actualizamos el DataGrid de clientes
                    FillGridCustomers();
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
           
        }

        //**************************************************************************
        //Nombre: button4_Click()
        //Descripción: 
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button4_Click(object sender, EventArgs e)
        {
            try
            {
                // Pedimos confirmación al usuario antes de proceder
                DialogResult status = MessageBox.Show("Desea habilitar la cuenta de los usuarios seleccionados?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (status.ToString() == "Yes")
                {
                    // Por cada fila del DataGrid de Clientes
                    foreach (DataGridViewRow row in grdCustomers.Rows)
                    {
                        // Comprovamos que la fila tenga valor
                        if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                        {
                            continue;
                        }
                        else
                        {
                            // Si la fila está seleccionada
                            if (row.Cells[0].Value.Equals(true))
                            {
                                // Hacemos la llamada al WebService para habilitar al cliente
                                Models.Customers DataContextCustomers = new Models.Customers();
                                string Resstatus = DataContextCustomers.UpdateStatusAccount(Convert.ToInt32(row.Cells[1].Value), 0);

                                //Mostramos el resultado de la operación
                                MessageBox.Show(Resstatus, "Resultado", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                    }
                    // Actualizamos el DataGrid de clientes
                    FillGridCustomers();
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddProvinces()
        //Descripción: Rellena los valores del COMBO de províncias
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddProvinces()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las províncias
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.msStateProvince[] SP = DataContextProduct.GetProvinces();
                cmbProvincias.Items.Clear();

                // Añadimos el primer registro en blanco.
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbProvincias.Items.Add(item);

                // Insertamos los valores en la COMBO
                foreach (DEOCIO_WS.msStateProvince mSP in SP)
                {
                    item = new ComboboxItem();
                    item.Text = mSP.Name;
                    item.Value = mSP.StateProvinceID;
                    cmbProvincias.Items.Add(item);
                }
                cmbProvincias.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje.
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena los valores del COMBO de Categorías
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddchkCategories()
        {
            try
            {
                //Hacemos la consulta al WebService para recuperar las categorías
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();

                chkCategories.Items.Clear();

                // Insertamos los valores en la COMBO.
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    ComboboxItem item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    chkCategories.Items.Add(item);
                }
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: addRolesToEditCmb()
        //Descripción: Inserta los valores de los Roles de la BBDD en el Combo de Roles
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void addRolesToEditCmb()
        {
            try
            {
                // Hacemos la llamada al Web Service
                Models.Customers DataContextCustomers = new Models.Customers();
                DEOCIO_WS.msRole[] Roles = DataContextCustomers.GetRoles();

                // Eliminamos todos los valores de la COMBO
                RolesToEditCmb.Items.Clear();

                //Insertamos el primer valor en blanco
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "0";
                RolesToEditCmb.Items.Add(item);

                // Por cada valor devuelto del WebService insertamos los valores en la COMBO
                foreach (msRole Rol in Roles)
                {
                    item = new ComboboxItem();
                    item.Text = Rol.Nombre;
                    item.Value = Rol.IdRole;
                    RolesToEditCmb.Items.Add(item);
                }

                // Seleccionamos el valor en blanco.
                RolesToEditCmb.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: btnCrear_Click()
        //Descripción: Botón para actualizar los datos del usuario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnCrear_Click(object sender, EventArgs e)
        {
            try
            {
                if ((txtNombre1.Text.Trim() != "") && (txtPrimerApellido1.Text.Trim() != "") && (txtSegundoApellido1.Text.Trim() != "")
                        && (txtDNI1.Text.Trim() != "") && (txtEmail1.Text.Trim() != "") && (txtPassword.Text.Trim() != "") && (txtDirLineOne.Text.Trim() != "") && (txtCp.Text.Trim() != "")
                        && (txtCity.Text.Trim() != "") && (TxtTelf.Text.Trim() != "") && (txtMovil.Text.Trim() != ""))
                {

                    // Validamos que las dos claves introducidas sean iguales.
                    if (txtPassword.Text.Trim() != txtConfirmPassword.Text.Trim())
                    {
                        // Si no lo son, mostramos mensaje y abortamos.
                        MessageBox.Show("Las contraseñas introducidas no son iguales", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    Models.Customers DataContextCustomers = new Models.Customers();
                    CustomerDataInfo CI = new CustomerDataInfo();
                    CI.FirstName = txtNombre1.Text;
                    CI.CustomerID = Convert.ToInt32(IdCustomer.Text);
                    CI.FirstName = txtNombre1.Text;
                    CI.MiddleName = txtPrimerApellido1.Text;
                    CI.LastName = txtSegundoApellido1.Text;
                    CI.DNI = txtDNI1.Text;
                    CI.Email = txtEmail1.Text;
                    CI.PasswordHash = txtPassword.Text;
                    CI.AddressLine = txtDirLineOne.Text;
                    CI.PostalCode = txtCp.Text;
                    CI.City = txtCity.Text;
                    CI.AddressID = Convert.ToInt32(IdAddress.Text);
                    CI.Phone = TxtTelf.Text;
                    CI.Mobile = txtMovil.Text;

                    // Cogemos el valor del COMBO de províncias, y validamos que no esté en blanco
                    int cmbProv = 0;
                    string ProvinceId = "";
                    if ((cmbProvincias.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbProv = Convert.ToInt32((cmbProvincias.SelectedItem as ComboboxItem).Value);
                    if (cmbProv.Equals(0))
                    {
                        // Si no se ha seleccionado ningún valor, mostramos mensaje y aboratmos.
                        MessageBox.Show("Debe seleccionar una província", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    else
                    {
                        ProvinceId = (cmbProvincias.SelectedItem as ComboboxItem).Value.ToString();
                    }

                    CI.StateProvinveID = ProvinceId;

                    // Cogemos el valor del COMBO de Roles, y validamos que no esté en blanco
                    int cmbRole = 0;
                    int Role = 0;
                    if ((RolesToEditCmb.SelectedItem as ComboboxItem).Value.ToString().Trim() != "") cmbRole = Convert.ToInt32((RolesToEditCmb.SelectedItem as ComboboxItem).Value);
                    if (cmbRole.Equals(0))
                    {
                        // Si no se ha seleccionado ningún valor, mostramos mensaje y aboratmos
                        MessageBox.Show("Debe seleccionar un Rol para este usuario", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    else
                    {
                        Role = Convert.ToInt32((RolesToEditCmb.SelectedItem as ComboboxItem).Value);
                    }

                    // Cogemos el valor del Check List de Categorías, y validamos que no esté en blanco
                    List<int> Categorias = new List<int>();
                    foreach (object itemChecked in chkCategories.CheckedItems)
                    {
                        Categorias.Add(Convert.ToInt32((itemChecked as ComboboxItem).Value));
                    }

                    if (Categorias.Count.Equals(0))
                    {
                        // Si no se ha seleccionado ningún valor, mostramos mensaje y aboratmos
                        MessageBox.Show("Debe seleccionar al menos una categoría", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    int cuentahabilitada = 0;
                    if (chkHabilitada.Checked) cuentahabilitada = 0;
                    else cuentahabilitada = 1;
                    CI.IsLockedOut = cuentahabilitada;
                    CI.Role = Role;
                    CI.Suffix = cmbSufijo.Text;
                    CI.Categories = Categorias.ToArray();

                    CustomerDataInfo RecuperaCI = new CustomerDataInfo();
                    RecuperaCI = DataContextCustomers.UpdateCustomerInfo(CI);
                    FillInfo(RecuperaCI);
                    FillGridCustomers();
                    tabControl1.SelectedIndex = 0;
                    EditMode(false);
                    CleanForm();
                    MessageBox.Show("Usuario modificado correctamente", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    
                }
                else
                {
                    // Si no se han rellenado todos los campos del formulario mostramos mensaje y aboratmos.
                    MessageBox.Show("Debe rellenar todos los datos del formulario.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error durante la llamada al WebService mostramos mensaje y abortamos.
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        //**************************************************************************
        //Nombre: btnCrear_Click()
        //Descripción: Botón para actualizar los datos del usuario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button3_Click_1(object sender, EventArgs e)
        {
             try
            {
                int index = -1;
                int contador = 0;
                foreach (DataGridViewRow row in grdCustomers.Rows)
                {
                    if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                    {
                        continue;
                    }
                    else
                    {
                        if (row.Cells[0].Value.Equals(true))
                        {
                            contador++;
                            index = Convert.ToInt32(row.Cells[1].Value);
                        }
                    }
                }

                // Si solo se ha seleccionado una oferta procedemos a mostrar su información
                if (contador == 1)
                {
                    // Hacemos la llamada al Web Service
                    Models.Customers DataContextCustomers = new Models.Customers();
                    CustomerDataInfo CI = new CustomerDataInfo();
                    CI = DataContextCustomers.RecoveryCustomerInfoById(index);
                    FillInfo(CI);

                    EditMode(false);
                    tabControl1.SelectedIndex = 1;

                }
                else
                {
                    MessageBox.Show("Seleccione solo una oferta para ampliar su información", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: btnCrear_Click()
        //Descripción: Botón para actualizar los datos del usuario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void FillInfo(CustomerDataInfo CI)
        {
            IdCustomer.Text = CI.CustomerID.ToString();
            txtNombre1.Text = CI.FirstName;
            txtPrimerApellido1.Text = CI.MiddleName;
            txtSegundoApellido1.Text = CI.LastName;
            txtDNI1.Text = CI.DNI;
            txtEmail1.Text = CI.Email;
            txtPassword.Text = CI.PasswordHash;
            txtConfirmPassword.Text = CI.PasswordHash;
            txtDirLineOne.Text = CI.AddressLine;
            txtCp.Text = CI.PostalCode;
            txtCity.Text = CI.City;
            IdAddress.Text = CI.AddressID.ToString();

            if (CI.IsLockedOut == 0) chkHabilitada.Checked = true;
            else chkHabilitada.Checked = false;

            // Seleccionamos el valor del combo de províncias según la província recuperada
            int count = 0;
            foreach (ComboboxItem CIM in cmbProvincias.Items)
            {
                if (CIM.Value.ToString().Trim() == CI.StateProvinveID.ToString().Trim()) cmbProvincias.SelectedIndex = count;
                count++;
            }

            // Seleccionamos el valor del combo de Roles según la província recuperada
            count = 0;
            foreach (ComboboxItem CIM in RolesToEditCmb.Items)
            {
                if (CIM.Value.ToString().Trim() == CI.Role.ToString().Trim()) RolesToEditCmb.SelectedIndex = count;
                count++;
            }


            List<int> Index = new List<int>();
            foreach (int CT in CI.Categories)
            {
                count = 0;
                foreach (ComboboxItem C in chkCategories.Items)
                {
                    if (C.Value.Equals(CT))
                    {
                        Index.Add(count);
                    }
                    count++;
                }
            }

            foreach (int Ind in Index)
            {
                chkCategories.SetItemCheckState(Ind, CheckState.Checked);
            }

            TxtTelf.Text = CI.Phone;
            txtMovil.Text = CI.Mobile;
        }

        //**************************************************************************
        //Nombre: EditMode()
        //Descripción: Habilita o Deshabilita el formulario de edición.
        //@params [bool Enable]
        //@return [None]
        //**************************************************************************
        private void EditMode(bool Enable)
        {
            txtNombre1.ReadOnly = Enable;
            txtPrimerApellido1.ReadOnly = Enable;
            txtSegundoApellido1.ReadOnly = Enable;
            txtDNI1.ReadOnly = Enable;
            txtPassword.ReadOnly = Enable;
            txtConfirmPassword.ReadOnly = Enable;
            txtDirLineOne.ReadOnly = Enable;
            txtCp.ReadOnly = Enable;
            txtCity.ReadOnly = Enable;
            TxtTelf.ReadOnly = Enable;
            txtMovil.ReadOnly = Enable;
            RolesToEditCmb.Enabled = !Enable;
            cmbProvincias.Enabled =  !Enable;
            cmbSufijo.Enabled = !Enable;
            panel2.Enabled = Enable;
            panel1.Enabled = !Enable;
            chkCategories.Enabled = !Enable;
        }

        //**************************************************************************
        //Nombre: CleanForm()
        //Descripción: Limpiar el formulario de edicion
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void CleanForm()
        {
            EditMode(true);
            IdCustomer.Text = "";
            txtNombre1.Text = "";
            txtPrimerApellido1.Text = "";
            txtSegundoApellido1.Text = "";
            txtDNI1.Text = "";
            txtEmail1.Text = "";
            txtPassword.Text = "";
            txtConfirmPassword.Text = "";
            txtDirLineOne.Text = "";
            txtCp.Text = "";
            txtCity.Text = "";
            IdAddress.Text = "";
            TxtTelf.Text = "";
            txtMovil.Text = "";
            cmbProvincias.SelectedIndex = -1;
            RolesToEditCmb.SelectedIndex = -1;
        }

        //**************************************************************************
        //Nombre: btnCancelar_Click()
        //Descripción: Deshace los cambios del formulario de edición.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnCancelar_Click(object sender, EventArgs e)
        {
            CleanForm();
            tabControl1.SelectedIndex = 0;
        }


    }
}
