﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DEOCIOAPP.Forms
{
    public partial class Login : Form
    {

         #region "Propiedades"

        private bool _Authenticated = false;
        private int _role = 0;

        public bool Authenticated
        {
            get { return _Authenticated; }
            set { _Authenticated = value; }
        }

        public int role
        {
            get { return _role; }
            set { _role = value; }
        }
        
        private string _Username = "";

        public string Username
        {
            get { return _Username; }
            set { _Username = value; }
        }

        #endregion

        // Registra el número de intentos.
        private int LoginRetryCounter = 0;

        //**************************************************************************
        //Nombre: Login()
        //Descripción: Inicializa el formulario
        //@params [None]
        //@return [None]
        //**************************************************************************
        public Login()
        {
            InitializeComponent();
            tbUsername.Focus();
        }

        //**************************************************************************
        //Nombre: btnCancel_Click()
        //Descripción: Cierra el formulario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnCancel_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        //**************************************************************************
        //Nombre: btnLogin_Click()
        //Descripción: Comprueba que el usuario y la contraseña no esten vacías
        //             Valida el usuario y la contraseña en la base de datos. --> si es ok continua
        //             Si no es Ok, permite dos intentos más.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnLogin_Click(object sender, EventArgs e)
        {
            EnableDisableButtons(false);
            FncLogin();
        }

        //**************************************************************************
        //Nombre: FncLogin()
        //Descripción: Valida si los datos introducidos en el formulario son correctos
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void FncLogin()
        {
            try
            {
                if (LoginRetryCounter < 2)
                {
                    if (tbPassword.Text.Length > 0 && tbUsername.Text.Length > 0)
                    {
                        // llamamos a la función para autenticar al usuario
                        string returncode = UserAuthenticated(tbUsername.Text, tbPassword.Text);
                        bool IsNum = Int32.TryParse(returncode, out _role);
                        if (IsNum)
                        {
                            if (role > 1)
                            {
                                Authenticated = true;
                                this.Close(); // Cierra este formulario pero no sale de la aplicación
                            }
                            else if (role.Equals(1))
                            {
                                MessageBox.Show("NO dispone de permisos para acceder a la aplicación.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                LoginRetryCounter++; // Incrementa al contador de reintentos
                                EnableDisableButtons(true);
                            }
                            else
                            {
                                MessageBox.Show(returncode);
                                LoginRetryCounter++; // Incrementa al contador de reintentos
                                EnableDisableButtons(true);
                            }
                        }
                        else
                        {
                            MessageBox.Show(returncode);
                            LoginRetryCounter++; // Incrementa al contador de reintentos
                            EnableDisableButtons(true);
                        }
                    }
                    else // El usuario o la clave estan vacios.
                    {
                        MessageBox.Show("Debe introducir un nombre de usuario y contraseña", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        LoginRetryCounter++;
                        EnableDisableButtons(true);

                    }
                }
                else // Demasiados intentos, se cierra la aplicación.
                {
                    MessageBox.Show("Ha excedido del número máximo de intentos.\nContacte con el Administrador para más información.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Application.Exit();
                }
            }
            catch
            {
                MessageBox.Show("Error, vuelva a intentarlo...", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                EnableDisableButtons(true);
            }
        }

        //**************************************************************************
        //Nombre: UserAuthenticated()
        //Descripción: Existe el usuario? Si es así, la clave es correcta?
        //@params [string p, string p_2]
        //@return [string]
        //**************************************************************************
        private string UserAuthenticated(string p, string p_2)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            string status = WS.ValidateUser(p.Trim(), p_2.Trim());

            return status;
            
        }

        //Habilita o deshabilita los botones del formulario.
        //**************************************************************************
        //Nombre: EnableDisableButtons()
        //Descripción: Habilita o deshabilita los botones Acceder y cancelar
        //@params [bool ED]
        //@return [None]
        //**************************************************************************
        private void EnableDisableButtons(bool ED)
        {
            btnCancel.Enabled = ED;
            btnLogin.Enabled = ED;
            btnRecovery.Enabled = ED;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            try{
                EnableDisableButtons(false);
                if (tbUsername.Text.Trim() == "")
                {
                    MessageBox.Show("Debe introducir una dirección de correo válida para recuperar su clave", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    EnableDisableButtons(true);
                    return;
                }
                else
                {
                    DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
                    string status = WS.RecoveryPassword(tbUsername.Text.Trim());
                    if (status == "OK") MessageBox.Show("Su clave ha sido enviada a su dirección de correo", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else  MessageBox.Show(status, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    EnableDisableButtons(true);
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                EnableDisableButtons(true);
            }
        }

    }
}
