﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DEOCIOAPP.Models;
using DEOCIOAPP.DEOCIO_WS;


// Formulario  de envío de correos.
namespace DEOCIOAPP.Forms
{
    public partial class SendMail : Form
    {
        //**************************************************************************
        //Nombre: SendMail()
        //Descripción: Inicializa el formulario
        //@params [None]
        //@return [None]
        //**************************************************************************
        public SendMail()
        {
            InitializeComponent();

            label7.Parent = pictureBox1;
            label7.BackColor = Color.Transparent;

            //Rellena los datos del DataGrid de Clientes
            addRolesToCmb();
            //Rellena los datos del Combo de Roles
            AddComboCategories();
            //Rellena los datos del DataGrid de ofertas
            FillGrid();
            //Rellena los datos del Combo de categorías.
            FillGridCustomers();

        }

        //**************************************************************************
        //Nombre: FillGrid()
        //Descripción: Rellena el DataGird con los valores devueltos del WebService
        //@params [None]
        //@return [None]
        //**************************************************************************
        public void FillGrid()
        {
            try
            {
                // Limpiamos todas las filas del Grid de ofertas
                grdOffers.Rows.Clear();

                // Analizamos el formulario de "Consulta"
                string ProdName = null;
                string ProdNumber = null;
                string ProdDesc = null;
                int? Category = null;

                if (txtProdName.Text.Trim() != "") ProdName = txtProdName.Text.Trim();
                if (txtProdNum.Text.Trim() != "") ProdNumber = txtProdNum.Text.Trim();
                if (txtDescShort.Text.Trim() != "") ProdDesc = txtDescShort.Text.Trim();

                if (cmbCategories.SelectedIndex != -1)
                {
                    if ((cmbCategories.SelectedItem as ComboboxItem).Text != "")
                    {
                        Category = Convert.ToInt32((cmbCategories.SelectedItem as ComboboxItem).Value);
                    }
                }

                //Hacemos la llamada al WebService para cargar las ofertas insertadas.
                Models.Products DataContextProduct = new Models.Products();
                ProductModel[] PM = DataContextProduct.GetProducts(ProdName, ProdNumber, ProdDesc, Category);

                // Por cada oferta devuelta cargamos los valores en el DataGrid
                foreach (ProductModel P in PM)
                {
                    string[] content = new string[] { "false", P.ProductID.ToString(), P.Name, P.ProductNumber, P.ProductShortDesc, P.SellStartDate.ToString(), P.SellEndDate.ToString(), Math.Round(P.OfferPrice,2).ToString() + "€" };
                    int Index = grdOffers.Rows.Add(content);


                    // Si la oferta está deshabilitada pintamos el fondo de la fila de color gris
                    if (P.Status.Equals(1))
                    {
                        grdOffers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGray;
                    }
                    // Si la oferta está habilitada, pintamos el fondo de la fila de color verde
                    else
                    {
                        grdOffers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGreen;
                    }
                }
            }
            catch
            {
                MessageBox.Show("Se ha producido un error al recuperar los datos del servidor. Vuelva a intentarlo pasados unos minutos", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: AddComboCategories()
        //Descripción: Rellena el combo de categorías con los valores de la base de datos
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void AddComboCategories()
        {
            try
            {
                // Hacemos la llamada al WebService
                Models.Products DataContextProduct = new Models.Products();
                DEOCIO_WS.dtProductSubCategory[] CAT = DataContextProduct.ProductSubCategory_GetAll();

                // Eliminamos todos los valores de la COMBO
                cmbCategories.Items.Clear();

                //Insertamos el primer valor en blanco
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "";
                cmbCategories.Items.Add(item);

                // Por cada valor devuelto del WebService insertamos los valores en la COMBO
                foreach (DEOCIO_WS.dtProductSubCategory C in CAT)
                {
                    item = new ComboboxItem();
                    item.Text = C.Name;
                    item.Value = C.ProductSubCategoryId;
                    cmbCategories.Items.Add(item);
                }

                // Seleccionamos el registro en blanco.
                cmbCategories.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: addRolesToCmb()
        //Descripción: Inserta los valores de los Roles de la BBDD en el Combo de Roles
        //@params [None]
        //@return [None]
        //**************************************************************************
        private void addRolesToCmb()
        {
            try
            {
                // Hacemos la llamada al Web Service
                Models.Customers DataContextCustomers = new Models.Customers();
                DEOCIO_WS.msRole[] Roles = DataContextCustomers.GetRoles();

                // Eliminamos todos los valores de la COMBO
                cmbRoles.Items.Clear();

                //Insertamos el primer valor en blanco
                ComboboxItem item = new ComboboxItem();
                item.Text = "";
                item.Value = "0";
                cmbRoles.Items.Add(item);

                // Por cada valor devuelto del WebService insertamos los valores en la COMBO
                foreach (msRole Rol in Roles)
                {
                    item = new ComboboxItem();
                    item.Text = Rol.Nombre;
                    item.Value = Rol.IdRole;
                    cmbRoles.Items.Add(item);
                }

                // Seleccionamos el valor en blanco.
                cmbRoles.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //**************************************************************************
        //Nombre: FillGridCustomers()
        //Descripción: Rellena los datos del DataGrid de clientes en base al filtro introducido
        //@params [None]
        //@return [None]
        //**************************************************************************
        public void FillGridCustomers()
        {
            // Analizamos los valores del formulario "Filtrar Clientes"
            string Name = null;
            string MiddleName = null;
            string LastName = null;
            string DNI = null;
            string eMail = null;
            int? Role = null;
            int CmbRole = 0;

            if (txtNombre.Text.Trim() != "") Name = txtNombre.Text.Trim();
            if (txtPrimerApellido.Text.Trim() != "") MiddleName = txtPrimerApellido.Text.Trim();
            if (txtSegundoApellido.Text.Trim() != "") LastName = txtSegundoApellido.Text.Trim();
            if (txtDNI.Text.Trim() != "") DNI = txtDNI.Text.Trim();
            if (txtEmail.Text.Trim() != "") eMail = txtEmail.Text.Trim();

            if (cmbRoles.SelectedItem != null) CmbRole = Convert.ToInt32((cmbRoles.SelectedItem as ComboboxItem).Value);
            if (CmbRole > 0) Role = CmbRole;

            // Hacemos la llamada al WebService con los datos del filtro
            Models.Customers DataContextCustomers = new Models.Customers();
            dtCustomer_RecoveryAllResult[] CU = DataContextCustomers.GetAllCustomers(Name, MiddleName, LastName, DNI, eMail, Role);

            // Limpiamos las filas del DataGrid de Clientes
            grdCustomers.Rows.Clear();

            // Por cada valor devuelto por el WebService, insertamos los valores en el DataGrid de Clientes.
            foreach (dtCustomer_RecoveryAllResult C in CU)
            {
                if (C.EmailPromotion == 1)
                {
                    string[] content = new string[] { "false", C.CustomerID.ToString(), C.FirstName, C.MiddleName, C.LastName, C.Nombre, C.DNI, C.Email, C.Phone, C.Mobile };

                    DataGridViewCellStyle style = new DataGridViewCellStyle();

                    int Index = grdCustomers.Rows.Add(content);

                    // Si el usuario está bloqueado pintamos el fondo de la fila de color gris
                    if (C.IsLockedOut.Equals(1))
                    {
                        grdCustomers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGray;
                    }
                    // Si el usuario está habilitado, pintamos el fondo de la fila de color verde
                    else
                    {
                        grdCustomers.Rows[Index].DefaultCellStyle.BackColor = Color.LightGreen;
                    }
                }
            }
        }

        //**************************************************************************
        //Nombre: btnBuscar_Click()
        //Descripción: Actualiza los datos del DataGrid de clientes con el filtro aplicado
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnBuscar_Click(object sender, EventArgs e)
        {
            FillGridCustomers();
        }

        //**************************************************************************
        //Nombre: button1_Click()
        //Descripción: Actualiza los datos del DataGrid de Ofertas con el filtro aplicado
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            FillGrid();
        }

        //**************************************************************************
        //Nombre: btnCerrar_Click()
        //Descripción: Cierra el formulario
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnCerrar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //**************************************************************************
        //Nombre: btnEnviar_Click()
        //Descripción: 
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void btnEnviar_Click(object sender, EventArgs e)
        {
            panel1.Enabled = false;
            try
            {
                StringBuilder Destinatarios = new StringBuilder("");
                StringBuilder Ofertas = new StringBuilder("");

                DialogResult status = MessageBox.Show("Está seguro que desea enviar las ofertas seleccionadas a los destinatarios seleccionados?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (status.ToString() == "Yes")
                {
                    // Por cada clientes selleccionado: insertamos su dirección de correo en la variable destinatarios
                    foreach (DataGridViewRow row in grdCustomers.Rows)
                    {
                        if ((row.Cells[7].Size.IsEmpty) || (row.Cells[7].Value == null))
                        {
                            continue;
                        }
                        else
                        {
                            if (row.Cells[0].Value.Equals(true))
                            {
                                if (Destinatarios.Length.Equals(0))
                                {
                                    Destinatarios.Append(row.Cells[7].Value.ToString().Trim());
                                }
                                else
                                {
                                    Destinatarios.Append(",");
                                    Destinatarios.Append(row.Cells[7].Value.ToString().Trim());
                                }
                            }
                        }
                    }

                    // Por cada oferta seleccionada: la insertamos en el cuerpo del mensaje
                    foreach (DataGridViewRow row in grdOffers.Rows)
                    {
                        if ((row.Cells[1].Size.IsEmpty) || (row.Cells[1].Value == null))
                        {
                            continue;
                        }
                        else
                        {
                            if (row.Cells[0].Value.Equals(true))
                            {

                                Ofertas.Append("<p><B><font size=5>" + row.Cells[4].Value.ToString().Trim() + " por solo " + row.Cells[7].Value.ToString().Trim()  + "</font></B>");
                                Ofertas.Append("<br /> Haz click aquí: " + System.Configuration.ConfigurationSettings.AppSettings["OfferUrl"] + row.Cells[1].Value.ToString().Trim());
                                Ofertas.Append("</p><br /><br />");
                            }
                        }
                    }

                    // Si las dos variables tiene valor, Hacemos la llamada al webService para que realice el envío del mensaje.
                    if ((Destinatarios.ToString() != "") && (Ofertas.ToString() != ""))
                    {
                        Models.Products DataContextProducts = new Models.Products();
                        bool resultado = DataContextProducts.SendEmail(Destinatarios.ToString(), "Ofertas Deocio.es", Ofertas.ToString());
                        if (resultado)
                        {
                            MessageBox.Show("Ofertas enviadas correctamente", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            panel1.Enabled = true;
                        }
                        else
                        {
                            MessageBox.Show("Error al enviar las ofertas, vuelva a intentarlo pasados unos minutos", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            panel1.Enabled = true;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                panel1.Enabled = true;
            }
        }

        //**************************************************************************
        //Nombre: button2_Click()
        //Descripción: Envía todas las ofertas vigentes de forma desatendida a los clientes segun 
        //             sus categorías inscritas.
        //@params [object sender, EventArgs e]
        //@return [None]
        //**************************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            button2.Enabled = false;
            btnEnviar.Enabled = false;
            btnCerrar.Enabled = false;
            try
            {
                DialogResult status = MessageBox.Show("Está seguro que desea realizar el envío automático de ofertas por preferencia?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (status.ToString() == "Yes")
                {
                    Models.Products DataContextProducts = new Models.Products();
                    bool resultado = DataContextProducts.SendOffersByPreference();
                    if (resultado)
                    {
                        MessageBox.Show("Ofertas enviadas correctamente", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        MessageBox.Show("Error al enviar las ofertas, vuelva a intentarlo pasados unos minutos", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    button2.Enabled = true;
                    btnEnviar.Enabled = true;
                    btnCerrar.Enabled = true;
                }
            }
            catch (Exception ex)
            {
                // Si se ha producido algun error, mostramos mensaje y abortamos
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                button2.Enabled = true;
                btnEnviar.Enabled = true;
                btnCerrar.Enabled = true;
            }
            button2.Enabled = true;
            button1.Enabled = true;
            btnCerrar.Enabled = true;
        }
    }
}
