﻿using System;
using DEOCIOAPP.DEOCIO_WS;

// Clase Customers: Realiza el tratamiento de las llamadas al WebService.
namespace DEOCIOAPP.Models
{
    class Customers
    {
        //**************************************************************************
        //Nombre: GetAllCustomers()
        //Descripción: Devuelve la información de todos los clientes de la base de datos en base al filtro pasado por parámetros
        //@params [string firstName, string middleName, string lastName, string dNI, string email, int? role]
        //@return [dtCustomer_RecoveryAllResult[]]
        //**************************************************************************
        public dtCustomer_RecoveryAllResult[] GetAllCustomers(string firstName, string middleName, string lastName, string dNI, string email, int? role)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.RecoveryAllCustomers(firstName, middleName, lastName, dNI, email, role);
        }

        //**************************************************************************
        //Nombre: GetRoles()
        //Descripción: Devuelve los roles de usuario definidos en la base de datos.
        //@params [None]
        //@return [msRole[]]
        //**************************************************************************
        public msRole[] GetRoles()
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.GetAllUserRoles();
        }

        //**************************************************************************
        //Nombre: AddUser()
        //Descripción: Añade un nuevo usuario en la base de datos con los datos pasados por parámetros.
        //@params [string sufijo, string name, string primerapellido, string segundoapellido, string dni, string email, string password, string direccion, string cp, string poblacion, string provincia, string telefono, string movil, bool contrato, bool recibirinfo, int role, int[] Categorias]
        //@return [string]
        //**************************************************************************
        public string AddUser(string sufijo, string name, string primerapellido, string segundoapellido,
                              string dni, string email, string password, string direccion, string cp, string poblacion,
                              string provincia, string telefono, string movil, bool contrato, bool recibirinfo, int role, int[] Categorias)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.AddUser(sufijo, name, primerapellido, segundoapellido, dni, email, password, direccion, cp, poblacion, provincia, telefono, movil, contrato, recibirinfo, role, Categorias );
        }

        //**************************************************************************
        //Nombre: DelCustomer()
        //Descripción: Elimina el usuario de la base de datos con ID igual al pasado por parámetro,
        //@params [int CustomerId]
        //@return [string]
        //**************************************************************************
        public string DelCustomer(int CustomerId)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.DelCustomer(CustomerId);
        }

        //**************************************************************************
        //Nombre: UpdateStatusAccount()
        //Descripción: Actualiza el estado de la cuenta del usuario (bloquear \ desbloquear)
        //@params [int CustomerId, int StatusAccount]
        //@return [string]
        //**************************************************************************
        public string UpdateStatusAccount(int CustomerId, int StatusAccount)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.UpdateStatusAccount(CustomerId,StatusAccount);
        }

        //**************************************************************************
        //Nombre: RecoveryPassword()
        //Descripción: Recuperamos la clave de acceso por dirección de correo
        //@params [string email]
        //@return [string]
        //**************************************************************************
        public string RecoveryPassword(string email)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.RecoveryPassword(email);
        }

        //**************************************************************************
        //Nombre: RecoveryCustomerInfoById()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerDataInfo RecoveryCustomerInfoById(int IdCustomer)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.RecoveryCustomerDataInfoById(IdCustomer);
        }

        //**************************************************************************
        //Nombre: UpdateCustomerInfo()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerDataInfo UpdateCustomerInfo(CustomerDataInfo Customer)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.UpdateCustomerInfo(Customer);
        }
    }
}
