﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DEOCIOAPP.DEOCIO_WS;

// Clase Products: Realiza el tratamiento de las llamadas al WebService.
namespace DEOCIOAPP.Models
{
    class Products
    {
        //**************************************************************************
        //Nombre: AddProduct()
        //Descripción: Añade una nueva oferta en la base de datos.
        //@params [ProductModel P]
        //@return [bool]
        //**************************************************************************
        public bool AddProduct(ProductModel P)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.AddProduct(P);
        }

        //**************************************************************************
        //Nombre: GetProducts()
        //Descripción: Consulta las ofertas de la base de datos en base a unos parámetros de consulta
        //@params [string Name, string ProductNumber, string ShortDesc, int? ProductCategory]
        //@return [ProductModel[]]
        //**************************************************************************
        public ProductModel[] GetProducts(string Name, string ProductNumber, string ShortDesc, int? ProductCategory)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.GetOffers(Name, ProductNumber, ShortDesc, ProductCategory);
        }

        //**************************************************************************
        //Nombre: DeleteOffer()
        //Descripción: Elimina la oferta con ID pasado por parámetro
        //@params [int ProductId]
        //@return [string]
        //**************************************************************************
        public string DeleteOffer(int ProductId)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.DeleteOffer(ProductId);
        }

        //**************************************************************************
        //Nombre: GetProvinces()
        //Descripción: Devuelve las províncias definidas en la tabla msStateProvince 
        //@params [None]
        //@return [msStateProvince[]]
        //**************************************************************************
        public msStateProvince[] GetProvinces()
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.GetProvinces();
        }

        //**************************************************************************
        //Nombre: GetProductOnlyByID()
        //Descripción:  Devuelve la información de un producto con id pasado por parámetro
        //@params [int ProductID]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel GetProductOnlyByID(int ProductID)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.GetProductOnlyById(ProductID);
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_GetAll()
        //Descripción: Devuelve todas las categorías existentes en la tabla dtProductSubCategory
        //@params [None]
        //@return [dtProductSubCategory[]]
        //**************************************************************************
        public dtProductSubCategory[] ProductSubCategory_GetAll()
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.ProductSubCategory_GetAll();
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Delete(int ProductSubcategoryId)
        //Descripción: Elimina de la base de datos la categoría con ID pasado por parámetro
        //@params [int ProductSubcategoryId]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Delete(int ProductSubcategoryId)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.ProductSubCategory_Delete(ProductSubcategoryId);
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Update()
        //Descripción: Actualiza la información de la categoría con ID pasado por parámetro
        //@params [int ProductSubcategoryId, string Name]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Update(int ProductSubcategoryId, string Name)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.ProductSubCategory_Update(ProductSubcategoryId, Name);
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Insert()
        //Descripción: Inserta una nueva categoría en la base de datos
        //@params [string Name]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Insert(string Name)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.ProductSubCategory_Insert(Name);
        }

        //**************************************************************************
        //Nombre: SendEmail()
        //Descripción: Envia un correo con los datos pasados por parámetros.
        //@params [string destination, string Header, string Text]
        //@return [bool]
        //**************************************************************************
        public bool SendEmail(string destination, string Header, string Text)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.SendEmail(destination, Header, Text);
        }

        //**************************************************************************
        //Nombre: SendOffersByPreference()
        //Descripción: Envía de forma automática las ofertas a los clientes por preferéncias.
        //@params [None]
        //@return [bool]
        //**************************************************************************
        public bool SendOffersByPreference()
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.SendOffersByPreference();
        }

        //**************************************************************************
        //Nombre: GetAllOrders()
        //Descripción: Obtiene un array de todas las compras realizadas
        //@params [None]
        //@return [None]
        //**************************************************************************
        public CommandInfo[] GetAllOrders(int? SalesOrderID, DateTime? OrderDate, string ProductShortDesc, int? ProductSubcategoryID, string ProductNumber, string Email, string DNI)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.GetAllOrders(SalesOrderID, OrderDate, ProductShortDesc, ProductSubcategoryID, ProductNumber, Email, DNI);
        }

        //**************************************************************************
        //Nombre: UpdateProductStatus()
        //Descripción: Actualiza el estado de un producto
        //@params [Int32 IdProduct, Int32 Status]
        //@return [string]
        //**************************************************************************
        public string UpdateProductStatus(int ProductID, int Status)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.UpdateProductStatus(ProductID, Status);
        }

        //**************************************************************************
        //Nombre: GetOrderById()
        //Descripción: Obtiene un array de la orden por PK
        //@params [int OrderId]
        //@return [None]
        //**************************************************************************
        public CommandInfo[] GetOrderById(int OrderId)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.GetOrderById(OrderId);
        }

        //**************************************************************************
        //Nombre: UpdateProducts()
        //Descripción: Actualiza la información de una oferta en la base de datos.
        //@params [ProductModel PM]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel UpdateProducts(ProductModel PM)
        {
            DEOCIO_WS.DEOCIO_WSClient WS = new DEOCIO_WS.DEOCIO_WSClient();
            return WS.UpdateProducts(PM);
        }
    }
}
