﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using DEOCIO.Models;
using System.Text;
using System.Reflection;
using DEOCIO.DataModel.Products;

// Controlador Home
namespace DEOCIO.Controllers
{
    public class HomeController : Controller
    {
        // Definimos los servicios para interactuar con el modelo de datos
        public ProductService ProductServices;
        public CommonService CommonServices;
        
        protected override void Initialize(RequestContext requestContext)
        {
            // Inicializamos los servicios.
            if (ProductServices == null) { ProductServices = new ProductModels(); }
            if (CommonServices == null) { CommonServices = new CommonModels(); }
            base.Initialize(requestContext);
        }

        //**************************************************************************
        //Nombre:  [HttpPost]Search()
        //Descripción: Recibe los parámetros del formulario de búsqueda y muestra los resultados
        //@params [string txtPoblacion, string txtDistancia, string PrecioDesde, string PrecioHasta, string Desde, string Hasta, string Ordenarpor, int Category]
        //**************************************************************************
        [HttpPost]
        public ActionResult Search(string txtPoblacion, string txtDistancia, string PrecioDesde, string PrecioHasta,
                                  string Desde, string Hasta, string Ordenarpor, int Category)
        {
            // Validamos los datos recibidos.
            string txtPoblacion_null = null ;
            int? var_MaxLong_null = null;
            int var_MaxLong = 0;
            decimal? var_Price_D_null = null;
            decimal var_Price_D = 0;
            decimal? var_Price_H_null = null;
            decimal var_Price_H = 0;
            DateTime var_Desde;
            DateTime var_Hasta;

            int? var_Category = null;

            if (txtPoblacion != "")
            {
                txtPoblacion_null = txtPoblacion;
            }

            int.TryParse(txtDistancia.Replace("Km",""), out var_MaxLong);
            if (var_MaxLong > 0)
            {
                var_MaxLong_null = var_MaxLong;
            }

            decimal.TryParse(PrecioDesde.Replace("€",""), out var_Price_D);
            if (var_Price_D > 0)
            {
                var_Price_D_null = var_Price_D;
            }

            decimal.TryParse(PrecioHasta.Replace("€",""), out var_Price_H);
            if (var_Price_H > 0)
            {
                var_Price_H_null = var_Price_H;
            }

            if (Category != -1)
            {
                var_Category = Category;
            }

            DateTime.TryParse(Desde, out var_Desde);
            DateTime.TryParse(Hasta, out var_Hasta);

            // Realizamos la consula al modelo para recuperar los datos según filtro
            List<ProductModel> products = ProductServices.getProductsByQuery(txtPoblacion_null, var_MaxLong_null, var_Price_D_null, var_Price_H_null, var_Desde, var_Hasta, Ordenarpor, var_Category);

            // Obtenemos los valores del COMO Ordenar por.
            List<OrderBy> OrderDrop = CommonServices.getOrderByList();
            var selectlist = new SelectList(OrderDrop, "Value", "Text", "");
            ViewData["OrderBy"] = selectlist;

            //Obtenemos los valores del combo categorías.
            List<dtProductSubCategory> Categories = ProductServices.ProductSubCategory_GetAll().ToList();
            dtProductSubCategory PEmpty = new dtProductSubCategory();
            PEmpty.Name = "";
            PEmpty.ProductSubCategoryId = -1;
            Categories.Add(PEmpty);
            var selectlist2 = new SelectList(Categories, "ProductSubCategoryId", "Name", Category);
            ViewData["cats"] = selectlist2;

            ViewData["Desde"] = Desde;
            ViewData["Hasta"] = Hasta;
            ViewData["Poblacion"] = txtPoblacion;

            // Devolvemos la lista de ofertas aleatórias (Parte inferior)
            List<ProductModel> prodOthers = ProductServices.getRandomProducts();
            ViewData["prodRandom"] = prodOthers;

            // Devolvemos los productos
            ViewData["products"] = products;
            
            // Devolvemos finalmente la vista.
            return View("Index", products);
        }

        //**************************************************************************
        //Nombre:  GetFirsLargeImage()
        //Descripción: Devuelve la imagen marcada como primária de la oferta
        //@params [int ProductId]
        //**************************************************************************
        public ActionResult GetFirsLargeImage(int ProductId)
        {
            // Hacemos la consutla al modelo
            dtProductPhoto_RecoveryByIdResult[] ProductsPhotos = ProductServices.ProductPhoto_RecoveryById(ProductId);
            // Aplicamos Linq para garantizar la consulta.
            var ProductPhotoPrimary = from n in ProductsPhotos where n.Primary == 0 select n;

            byte[] LargeImage;
            string LargeImageType = "";

            foreach (dtProductPhoto_RecoveryByIdResult Ph in ProductPhotoPrimary)
            {
                LargeImage = Ph.ThumbNailPhoto.ToArray();
                LargeImageType = Ph.ThumbnailPhotoType.ToString();
                // Devolvemos la primera imagen primária
                return File(LargeImage, LargeImageType);
            }
            // Devolvemos null si algo falla.
            return null;
        }

        //**************************************************************************
        //Nombre:  GetThumbnailFile()
        //Descripción: Devuelve la imagen thumbnail con ID pasado por parámetro
        //@params [int PhotoId]
        //**************************************************************************
        public ActionResult GetThumbnailFile(int PhotoId)
        {
            // Hacemos la consulta al modelo de datos.
            dtProductPhoto[] ProductsPhotos = ProductServices.ProductPhoto_RecoveryByPhotoId(PhotoId).ToArray();

            byte[] ThumbnailImage;
            string LargeImageType = "";


            foreach (dtProductPhoto k in ProductsPhotos)
            {
                // Devolvemos la imagen
                ThumbnailImage = k.LargePhoto.ToArray();
                LargeImageType = k.LargePhotoType.ToString();
                return File(ThumbnailImage, LargeImageType);
            }
            // Devolvemos null si algo falla.
            return null;
        }

        //**************************************************************************
        //Nombre:  GetThumbnailsImage()
        //Descripción: Devuelve la imagen thumbnail del producto @ProductID
        //@params [int ProductId]
        //**************************************************************************
        public ActionResult GetThumbnailsImage(int ProductId)
        {
            // Hacemos la consulta al modelo de datos.
            dtProductPhoto_RecoveryByIdResult[] ProductsPhotos = ProductServices.ProductPhoto_RecoveryById(ProductId);

            byte[] ThumbnailImage;
            string LargeImageType = "";

            foreach (dtProductPhoto_RecoveryByIdResult k in ProductsPhotos)
            {
                ThumbnailImage = k.ThumbNailPhoto.ToArray();
                LargeImageType = k.LargePhotoType.ToString();
                // Devolvemos la imagen
                return File(ThumbnailImage, LargeImageType);
            }
            // Devolvemos null si algo falla.
            return null;
        }

        //**************************************************************************
        //Nombre:  Index()
        //Descripción: Muestra la información de la página inicial
        //@params [None]
        //**************************************************************************
        public ActionResult Index()
        {
            // Devuelve los valores del como Ordenar por.
            List<OrderBy> OrderDrop = CommonServices.getOrderByList();
            var selectlist = new SelectList(OrderDrop, "Value", "Text", "");
            ViewData["OrderBy"] = selectlist;

            // Devuelve los valores del combo categorías
            List<dtProductSubCategory> Categories = ProductServices.ProductSubCategory_GetAll().ToList();
            dtProductSubCategory PEmpty = new dtProductSubCategory();
            PEmpty.Name = "";
            PEmpty.ProductSubCategoryId = -1;
            Categories.Add(PEmpty);
            var selectlist2 = new SelectList(Categories, "ProductSubCategoryId", "Name", -1);
            ViewData["cats"] = selectlist2;

            ViewData["Desde"] = DateTime.Now;
            ViewData["Hasta"] = DateTime.Now.AddMonths(1);

            // Devuelve los valores de las ofertas
            List<ProductModel> products = ProductServices.getLastProducts();
            ViewData["products"] = products;

            // Devuelve los valores de las ofertas aleatórias (parte inferior)
            List<ProductModel> prodOthers = ProductServices.getRandomProducts();
            ViewData["prodRandom"] = prodOthers;
            ViewData["Poblacion"] = "";

            // Devuelve la vista.
            return View(products);
        }
    }
}
