﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using DEOCIO.Models;
using System.Web.Routing;
using DEOCIO.DataModel.Orders;
using DEOCIO.DataModel.Products;

// Controlador Offer
namespace DEOCIO.Controllers
{
    [Authorize]
    public class OfferController : Controller
    {
        // Definimos los servicios para interactuar con el modelo de datos
        public ProductService ProductServices;
        public IMembershipService MembershipService { get; set; }

        protected override void Initialize(RequestContext requestContext)
        {
            // Inicializamos los servicios.
            if (ProductServices == null) { ProductServices = new ProductModels(); }
            if (MembershipService == null) { MembershipService = new AccountMembershipService(); }
            base.Initialize(requestContext);
        }

        //**************************************************************************
        //Nombre: OfferDetail()
        //Descripción: Devuelve la información ampliada de la oferta
        //@params [int OfferId]
        //**************************************************************************
        public ActionResult OfferDetail(int OfferId)
        {
            // Hacemos la consulta al modelo de datos para obtener la información ampliada de la oferta
            CustomerInfo CI = MembershipService.RecoveryCustomerInfoById(HttpContext.User.Identity.Name.ToString());

            // Devolvemos las fotos vinculadas a la oferta
            List<dtProductPhoto_RecoveryByIdResult> ProductPhotos = ProductServices.ProductPhoto_RecoveryById(OfferId).ToList();
            ViewData["ProdPhotos"] = ProductPhotos;

            // Devolvemos la localización de la oferta
            dtProductLocation_RecoveryByProductIdResult Location = ProductServices.GetProductLocation(OfferId);
            ViewData["ProductLocation"] = Location;

            // Devolvemos la información de la oferta
            ProductModel product = ProductServices.getProductById(OfferId, CI.Customer.CustomerID);
            ViewData["product"] = product;

            // Devolvemos la vista.
            return View(product);
        }

        //**************************************************************************
        //Nombre: BuyOffer()
        //Descripción: Inicia el proceso de compra. Muestra el formulario de compra.
        //@params [int OfferId]
        //**************************************************************************
        public ActionResult BuyOffer(int OfferId)
        {
            // Devuelve los valores del COMBO "Cantidad a comprar"
            var selectlist = new SelectList(ProductServices.getmaxProductBuy(OfferId), 1);
            ViewData["MaxNumberProductsBuy"] = selectlist;

            // Devuvelve la información del usuario
            CustomerInfo CI = MembershipService.RecoveryCustomerInfoById(HttpContext.User.Identity.Name.ToString());
            ViewData["customer"] = CI;

            // Devuelve la información de la oferta.
            ProductModel product = ProductServices.getProductById(OfferId, CI.Customer.CustomerID);
            ViewData["product"] = product;

            // Devuelve la vista.
            return View(product);
        }

        //**************************************************************************
        //Nombre: ConfirmBuy()
        //Descripción: Realiza el proceso de compra.
        //@params [string NumProduct, string ProductID, string SelAddress, CustomerInfo CI]
        //**************************************************************************
        public ActionResult ConfirmBuy(string NumProduct, string ProductID, string SelAddress, CustomerInfo CI)
        {
            // Recuperamos de nuevo la información del cliente.
            CustomerInfo CustomerInf = MembershipService.RecoveryCustomerInfoById(HttpContext.User.Identity.Name.ToString());

            // Hacemos la llamada al modelo de datos para realizar la compra.
            string SalesId = ProductServices.BuyOffer(NumProduct, ProductID, SelAddress, CustomerInf);

            // Redireccionamos la petición a: Operacion Completa 
            RouteValueDictionary rvd = new RouteValueDictionary();
            rvd.Add("SalesId", SalesId);
            return RedirectToAction("OperationComplete", "Offer", rvd);
        }

        //**************************************************************************
        //Nombre: OperationComplete()
        //Descripción: Muestra la confirmación de la compra
        //@params [string SalesId]
        //**************************************************************************
        public ActionResult OperationComplete(string SalesId)
        {
            // Recupera la información de la orden de compra
            int IdSales = Convert.ToInt32(SalesId);
            ConfirmationBuy CB = ProductServices.getCommandInfo(IdSales);

            // Recupera la información del usuario\cliene que está validado.
            CustomerInfo CustomerInf = MembershipService.RecoveryCustomerInfoById(HttpContext.User.Identity.Name.ToString());

            // Si el usuario que hace la consutla no es el mismo que ha realizado la compra 
            if (CB.Customer.CustomerID.Equals(CustomerInf.Customer.CustomerID))
            {
                // Devuelve la vista.
                return View(CB);
            }
            else
            {
                // Devuelve la vista.
                return RedirectToAction("Index", "Home");
            }
        }

        //**************************************************************************
        //Nombre: Comments()
        //Descripción: Añade un comentario a la oferta
        //@params [string Commenttxt, int ProductId, string txtPuntuacion]
        //**************************************************************************
        [HttpPost]
        public ActionResult Comments(string Commenttxt, int ProductId, string txtPuntuacion)
        {
            // Recupera la información del usuario del modelo de datos.
            CustomerInfo CustomerInf = MembershipService.RecoveryCustomerInfoById(HttpContext.User.Identity.Name.ToString());
            
            // Hace la llamada al modelo de datos para insertar un nuevo comentario.
            string status = ProductServices.addComment(Commenttxt, ProductId, CustomerInf, txtPuntuacion);

            // Redirecciona la vista a la misma pantalla que estava.
            RouteValueDictionary rvd = new RouteValueDictionary();
            rvd.Add("OfferId", ProductId);
            return RedirectToAction("OfferDetail", "Offer", rvd);
        }

        //**************************************************************************
        //Nombre: MisCompras()
        //Descripción: Devuelve la lista de compras realizadas por el usuario.
        //@params [None]
        //**************************************************************************
        public ActionResult MisCompras()
        {
            // Recupera la información del usuario del modelo de datos.
            CustomerInfo CI = MembershipService.RecoveryCustomerInfoById(HttpContext.User.Identity.Name.ToString());
            
            // Recupera la lista de ordenes de compras realizada por el usuario
            List<dtSalesOrder_RecoveryByUserResult> OrdersByUser = ProductServices.MisCompras(CI);

            List<ConfirmationBuy> CBL = new List<ConfirmationBuy>();

            foreach (dtSalesOrder_RecoveryByUserResult sr in OrdersByUser)
            {
                CBL.Add(ProductServices.getCommandInfo(sr.SalesOrderID));
            }

            // Devuelve los valores y la vista.
            ViewData["OrdersByUser"] = CBL;
            return View();
        }
    }
}
