﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using DEOCIO.Models;
using System.IO;
using DEOCIO.DataModel.TablasMaestras.msStateProvince;
using DEOCIO.DataModel.Customers;
using DEOCIO.DataModel.TablasMaestras.msRoles;
using DEOCIO.DataModel.Products;
using System.Configuration;

namespace DEOCIO
{
    public class DEOCIO_WS : IDEOCIO_WS
    {
        //**************************************************************************
        //Nombre: ValidateUser()
        //Descripción: Valida el usuario i password pasados por parámetros
        //@params [string Email, string password]
        //@return [string]
        //**************************************************************************
        public string ValidateUser(string Email, string password)
        {
            DEOCIO.Models.AccountMembershipService AS = new DEOCIO.Models.AccountMembershipService();
            return AS.ValidateUser(Email, password);
        }

        //**************************************************************************
        //Nombre: AddProduct()
        //Descripción: Añade una nueva oferta en la base de datos.
        //@params [ProductModel P]
        //@return [bool]
        //**************************************************************************
        public bool AddProduct(ProductModel PM)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.AddNewProduct(PM);
        }

         //**************************************************************************
        //Nombre: GetOffers()
        //Descripción: Consulta las ofertas de la base de datos en base a unos parámetros de consulta
        //@params [string Name, string ProductNumber, string ShortDesc, int? ProductCategory]
        //@return [ProductModel[]]
        //**************************************************************************
        public List<ProductModel> GetOffers(string Name, string ProductNumber, string ShortDesc, int? ProductCategory)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            List<ProductModel> products = PS.getAllProducts(Name, ProductNumber, ShortDesc, ProductCategory);
            return products;
        }

        //**************************************************************************
        //Nombre: DeleteOffer()
        //Descripción: Elimina la oferta con ID pasado por parámetro
        //@params [int ProductId]
        //@return [string]
        //**************************************************************************
        public string DeleteOffer(int ProductID)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.DeleteProduct(ProductID);
        }

        //**************************************************************************
        //Nombre: GetProvinces()
        //Descripción: Devuelve las províncias definidas en la tabla msStateProvince 
        //@params [None]
        //@return [msStateProvince[]]
        //**************************************************************************
        public List<msStateProvince> GetProvinces()
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.msStateProvince_All();
        }

        //**************************************************************************
        //Nombre: GetProductOnlyByID()
        //Descripción:  Devuelve la información de un producto con id pasado por parámetro
        //@params [int ProductID]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel GetProductOnlyById(int ProductID)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.GetProductOnlyById(ProductID);
        }

        //**************************************************************************
        //Nombre: RecoveryAllCustomers()
        //Descripción: Devuelve la información de todos los clientes de la base de datos en base al filtro pasado por parámetros
        //@params [string firstName, string middleName, string lastName, string dNI, string email, int? role]
        //@return [dtCustomer_RecoveryAllResult[]]
        //**************************************************************************
        public dtCustomer_RecoveryAllResult[] RecoveryAllCustomers(string firstName, string middleName, string lastName, string dNI, string email, int? role)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.CustomerRecoveryAll(firstName, middleName, lastName, dNI, email, role);
        }

        //**************************************************************************
        //Nombre: GetAllUserRoles()
        //Descripción: Devuelve los roles de usuario definidos en la base de datos.
        //@params [None]
        //@return [msRole[]]
        //**************************************************************************
        public msRole[] GetAllUserRoles()
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.GetAllRoles();
        }

        //**************************************************************************
        //Nombre: AddUser()
        //Descripción: Añade un nuevo usuario en la base de datos con los datos pasados por parámetros.
        //@params [string sufijo, string name, string primerapellido, string segundoapellido, string dni, string email, string password, string direccion, string cp, string poblacion, string provincia, string telefono, string movil, bool contrato, bool recibirinfo, int role, int[] Categorias]
        //@return [string]
        //**************************************************************************
        public string AddUser(string sufijo, string name, string primerapellido, string segundoapellido,
                                          string dni, string email, string password, string direccion, string cp, string poblacion,
                                          string provincia, string telefono, string movil, bool contrato, bool recibirinfo, int role, IEnumerable<int> categorias)
        {
             DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
             return AM.CreateUser(sufijo, name, primerapellido, segundoapellido,
                                          dni, email, password, direccion, cp, poblacion,
                                          provincia, telefono, movil, contrato, recibirinfo, role, categorias);
        }

        //**************************************************************************
        //Nombre: DelCustomer()
        //Descripción: Elimina el usuario de la base de datos con ID igual al pasado por parámetro,
        //@params [int CustomerId]
        //@return [string]
        //**************************************************************************
        public string DelCustomer(int CustomerId)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.DeleteCustomer(CustomerId);
        }

        //**************************************************************************
        //Nombre: UpdateStatusAccount()
        //Descripción: Actualiza el estado de la cuenta del usuario (bloquear \ desbloquear)
        //@params [int CustomerId, int StatusAccount]
        //@return [string]
        //**************************************************************************
        public string UpdateStatusAccount(int CustomerId, int StatusAccount)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.UpdateStatusAccount(CustomerId, StatusAccount);
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_GetAll()
        //Descripción: Devuelve todas las categorías existentes en la tabla dtProductSubCategory
        //@params [None]
        //@return [dtProductSubCategory[]]
        //**************************************************************************
        public dtProductSubCategory[] ProductSubCategory_GetAll()
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.ProductSubCategory_GetAll();
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Delete(int ProductSubcategoryId)
        //Descripción: Elimina de la base de datos la categoría con ID pasado por parámetro
        //@params [int ProductSubcategoryId]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Delete(int ProductSubcategoryId)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.ProductSubCategory_Delete(ProductSubcategoryId);
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Update()
        //Descripción: Actualiza la información de la categoría con ID pasado por parámetro
        //@params [int ProductSubcategoryId, string Name]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Update(int ProductSubcategoryId, string Name)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.ProductSubCategory_Update(ProductSubcategoryId, Name);
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Insert()
        //Descripción: Inserta una nueva categoría en la base de datos
        //@params [string Name]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Insert(string Name)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.ProductSubCategory_Insert(Name);
        }

        //**************************************************************************
        //Nombre: GetAllOrders()
        //Descripción: Obtiene un array de todas las compras realizadas
        //@params [None]
        //@return [None]
        //**************************************************************************
        public CommandInfo[] GetAllOrders(int? SalesOrderID, DateTime? OrderDate, string ProductShortDesc, int? ProductSubcategoryID, string ProductNumber, string Email, string DNI)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.GetAllOrders(SalesOrderID, OrderDate, ProductShortDesc, ProductSubcategoryID, ProductNumber, Email, DNI);
        }

        //**************************************************************************
        //Nombre: RecoveryPassword()
        //Descripción: Recuperamos la clave de acceso por dirección de correo
        //@params [string email]
        //@return [string]
        //**************************************************************************
        public string RecoveryPassword(string email)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.RecoveryPassword(email);
        }

         //**************************************************************************
        //Nombre: UpdateProductStatus()
        //Descripción: Actualiza el estado de un producto
        //@params [Int32 IdProduct, Int32 Status]
        //@return [string]
        //**************************************************************************
        public string UpdateProductStatus(int ProductID, int Status)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.UpdateProductStatus(ProductID, Status);
        }

        //**************************************************************************
        //Nombre: SendEmail()
        //Descripción: Envia un correo con los datos pasados por parámetros.
        //@params [string destination, string Header, string Text]
        //@return [bool]
        //**************************************************************************
        public bool SendEmail(string destination, string Header, string Text)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.SendEmail(destination, Header, Text);
        }

        //**************************************************************************
        //Nombre: SendOffersByPreference()
        //Descripción: Envía de forma automática las ofertas a los clientes por preferéncias.
        //@params [None]
        //@return [bool]
        //**************************************************************************
        public bool SendOffersByPreference()
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();

            // Recuperamos todas las categorías de la base de datos.
            dtProductSubCategory[] PC = PS.ProductSubCategory_GetAll();

            // Por cada categoría recuperada
            foreach (dtProductSubCategory PsC in PC)
            {
                //Recuperamos todas las ofertas que pertenezcan a esa categoría.
                dtProduct[] Products = PS.getProductsByCategory(PsC.ProductSubCategoryId);
                
                // Recuperamos todos los clientes que esten subscritos a esa categoría.
                dtCustomer[] Customers = AM.getCustomersByCategory(PsC.ProductSubCategoryId);

                StringBuilder Destinatarios = new StringBuilder("");
                StringBuilder Ofertas = new StringBuilder("");

                // Elaboramos la lista de de destinatarios
                foreach (dtCustomer Customer in Customers)
                {
                    if (Destinatarios.Length.Equals(0))
                    {
                        Destinatarios.Append(Customer.Email.Trim());
                    }
                    else
                    {
                        Destinatarios.Append(",");
                        Destinatarios.Append(Customer.Email.Trim());
                    }
                }

                // Elaboramos la lista de ofertas a enviar para cada categoría.
                foreach (dtProduct Product in Products)
                {
                    Ofertas.Append("<p><B><font size=5>" + Product.ProductShortDesc.ToString().Trim() + " por solo " + Math.Round(Product.OfferPrice,2).ToString()+"€</font></B>");
                    Ofertas.Append("<br /> Haz click aquí: " + ConfigurationManager.AppSettings["OfferUrl"] + Product.ProductID.ToString().Trim());
                    Ofertas.Append("</p><br /><br />");
                }

                // Se realiza el envío del correo.
                if((Destinatarios.ToString().Trim() != "") && (Ofertas.ToString().Trim() != ""))
                {
                    bool status = SendEmail(Destinatarios.ToString(), "Boletín Ofertas Deocio.es Categoría: " + PsC.Name, Ofertas.ToString());
                    if (status == false) return false;
                }
            }
            return true;
        }

         //**************************************************************************
        //Nombre: UpdateCustomerInfo()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerDataInfo UpdateCustomerInfo(CustomerDataInfo Customer)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.UpdateCustomerInfo(Customer);
        }

        //**************************************************************************
        //Nombre: RecoveryCustomerById()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerDataInfo RecoveryCustomerDataInfoById(int IdCustomer)
        {
            DEOCIO.Models.AccountMembershipService AM = new DEOCIO.Models.AccountMembershipService();
            return AM.RecoveryCustomerDataInfoById(IdCustomer);
        }

        //**************************************************************************
        //Nombre: GetOrderById()
        //Descripción: Obtiene un array de la orden por PK
        //@params [int OrderId]
        //@return [None]
        //**************************************************************************
        public CommandInfo[] GetOrderById(int OrderId)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.GetOrderById(OrderId);
        }

        //**************************************************************************
        //Nombre: UpdateProducts()
        //Descripción: Actualiza la información de una oferta en la base de datos.
        //@params [ProductModel PM]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel UpdateProducts(ProductModel PM)
        {
            DEOCIO.Models.ProductModels PS = new DEOCIO.Models.ProductModels();
            return PS.UpdateProducts(PM);
        }


    }
}
