﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Globalization;
using System.Linq;
using System.Web.Mvc;
using System.Web.Security;
using DEOCIO.DataModel.Customers;
using DEOCIO.DataModel.TablasMaestras.msStateProvince;
using DEOCIO.DataModel.TablasMaestras.msRoles;
using System.Transactions;
using System.Net.Mail;
using System.Net;

namespace DEOCIO.Models
{

    #region Models

    //**************************************************************************
    //Nombre: CustomerInfo()
    //Descripción: Modelo de datos utilitzado para contener la información del usuario\cliente
    //@params [None]
    //@return [None]
    //**************************************************************************
     public class CustomerInfo
    {
         public dtCustomer Customer  {get; set;}
         public dtAddress[] Addresses {get; set;}
    }

     public class CustomerDataInfo
     {
         public int CustomerID { get; set; }
         public string FirstName { get; set; }
         public string MiddleName { get; set; }
         public string LastName { get; set; }
         public string Suffix { get; set; }
         public string DNI { get; set; }
         public string Email { get; set; }
         public int EmailPromotion { get; set; }
         public string Phone { get; set; }
         public string Mobile { get; set; }
         public string PasswordHash { get; set; }
         public int Role { get; set; }
         public int IsApproved { get; set; }
         public System.Nullable<int> IsLockedOut { get; set; }
         public System.DateTime ModifiedDate { get; set; }
         public int AddressID { get; set; }
         public string AddressLine { get; set; }
         public string City { get; set; }
         public string StateProvinveID { get; set; }
         public string PostalCode { get; set; }
         public List<int> Categories {get; set;}
     }

     //**************************************************************************
     //Nombre: LogOnModel()
     //Descripción: Modelo de datos para el formulario de Validación de usuario
     //@params [None]
     //@return [None]
     //**************************************************************************
    public class LogOnModel
    {
        [Required]
        [Display(Name = "Nombre de usuario")]
        public string UserName { get; set; }

        [Required]
        [DataType(DataType.Password)]
        [Display(Name = "Clave")]
        public string Password { get; set; }

        [Display(Name = "Recordar?")]
        public bool RememberMe { get; set; }
    }

    //**************************************************************************
    //Nombre: RegisterModel()
    //Descripción: Modelo de datos del formulario de registro
    //@params [None]
    //@return [None]
    //**************************************************************************
    public class RegisterModel
    {
        [Required]
        [Display(Name = "Sufijo")]
        public string Sufijo { get; set; }

        [Required]
        [StringLength(50, MinimumLength = 3, ErrorMessage = "jj")]
        [Display(Name = "Nombre")]
        public string Name { get; set; }

        [Required]
        [Display(Name = "1º Apellido")]
        public string PrimerApellido { get; set; }

        [Required]
        [Display(Name = "2º Apellido")]
        public string SegundoApellido { get; set; }

        [Required]
        [Display(Name = "DNI")]
        public string DNI { get; set; }

        [Required]
        [Display(Name = "Tus preferencias")]
        public IEnumerable<int> Categorias { get; set; }

        [Required]
        [DataType(DataType.EmailAddress)]
        [ValidateEmail(ErrorMessage = "Se requiere una dirección de correo válida.")]
        [Display(Name = "Dirección de correo")]
        public string Email { get; set; }

        [Required]
        [ValidatePasswordLength]
        [DataType(DataType.Password)]
        [Display(Name = "Clave")]
        public string Password { get; set; }

        [DataType(DataType.Password)]
        [Display(Name = "Confirmar clave")]
        [Compare("Password", ErrorMessage = "La clave i la clave de confirmación no coinciden.")]
        public string ConfirmPassword { get; set; }

        [Required]
        [Display(Name = "Dirección")]
        public string Direccion { get; set; }

        [Required]
        [Display(Name = "Código Postal")]
        public string CP { get; set; }

        [Required]
        [Display(Name = "Población")]
        public string Poblacion { get; set; }

        [Required]
        [RegularExpression(@"^(?!.*(S)).*$",ErrorMessage="Debe seleccionar una província.")] 
        [Display(Name = "Província")]
        public string Provincia { get; set; }

        [Required]
        [Display(Name = "Teléfono")]
        public string Telefono { get; set; }

        [Required]
        [Display(Name = "Móvil")]
        public string Movil { get; set; }

        [Required]
        [Display(Name = "Desea recibir información?")]
        public bool RecibirInfo { get; set; }

        [Required]
        [Display(Name = "Contrato")]
        public bool Contrato { get; set; }

    }
    #endregion

    #region Services
    // Se definen los métodos que estaran disponibles des del resto de la aplicación

    public interface IMembershipService
    {
        int MinPasswordLength { get; }
        string ValidateUser(string userName, string password);
        string RecoveryPassword(string email);
        List<msStateProvince> msStateProvince_All();
        string CreateUser(string sufijo, string name, string primerapellido, string segundoapellido,
                          string dni, string email, string password, string direccion, string cp, string poblacion,
                          string provincia, string telefono, string movil, bool contrato, bool recibirinfo, int Role, IEnumerable<int> Categorias);
        CustomerInfo RecoveryCustomerInfoById(String IdCustomer);
        dtCustomer_RecoveryAllResult[] CustomerRecoveryAll(string firstName, string middleName, string lastName, string dNI, string email, int? role);
        msRole[] GetAllRoles();
        string DeleteCustomer(int CustomerId);
        string UpdateStatusAccount(int CustomerId, int StatusAccount);
        bool SendEmail(string destination, string Header, string Text);
        CustomerDataInfo RecoveryCustomerDataInfoById(int IdCustomer);
    }

    public class ValidateEmailAttribute : RegularExpressionAttribute
    {
        //**************************************************************************
        //Nombre: ValidateEmailAttribute()
        //Descripción: Valida que la dirección de correo introducida sea válida.
        //@params [None]
        //@return [None]
        //**************************************************************************
        public ValidateEmailAttribute() :
            base(@"^(([A-Za-z0-9]+_+)|([A-Za-z0-9]+\-+)|([A-Za-z0-9]+\.+)|([A-Za-z0-9]+\++))*[A-Za-z0-9]+@((\w+\-+)|(\w+\.))*\w{1,63}\.[a-zA-Z]{2,6}$") { }
    } 


    public class AccountMembershipService : IMembershipService
    {
        private readonly MembershipProvider _provider;

        public AccountMembershipService()
            : this(null)
        {
        }

        public AccountMembershipService(MembershipProvider provider)
        {
            _provider = provider ?? Membership.Provider;
        }

        public int MinPasswordLength
        {
            get
            {
                return _provider.MinRequiredPasswordLength;
            }
        }

       
        //**************************************************************************
        //Nombre: RecoveryCustomerInfoById()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerInfo RecoveryCustomerInfoById(String IdCustomer)
        {
            // Creamos una nueva instáncia de CustomerInfo
            CustomerInfo CustomerInformation = new CustomerInfo();

            // Creamos el DataContex de clientes
            dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();

            // Ejecutamos el stored procedure para recuperar la información de la base de datos.
            CustomerInformation.Customer = CustomersDataContext.dtCustomers_RecoveryCustomerByEmail(IdCustomer).FirstOrDefault();
            // Ejecutamos el stored procedure para recuperar la información de la base de datos.
            CustomerInformation.Addresses = CustomersDataContext.dtAddress_RecoveryByCustomerId(CustomerInformation.Customer.CustomerID).ToArray();
            // Retornamos la información recuperada
            return CustomerInformation;
        }

        //**************************************************************************
        //Nombre: RecoveryCustomerDataInfoById()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerDataInfo RecoveryCustomerDataInfoById(int IdCustomer)
        {
            // Creamos una nueva instáncia de CustomerInfo
           CustomerInfo CustomerInformation = new CustomerInfo();

            // Creamos el DataContex de clientes
           dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
            
            // Ejecutamos el stored procedure para recuperar la información de la base de datos.
           CustomerInformation.Customer = CustomersDataContext.dtCustomers_RecoveryByCustomerID(IdCustomer).FirstOrDefault();
           CustomerInformation.Addresses = CustomersDataContext.dtAddress_RecoveryByCustomerId(CustomerInformation.Customer.CustomerID).ToArray();

           CustomerDataInfo CDI = new CustomerDataInfo();
           CDI.CustomerID = CustomerInformation.Customer.CustomerID;
           CDI.FirstName = CustomerInformation.Customer.FirstName;
           CDI.MiddleName = CustomerInformation.Customer.MiddleName;
           CDI.LastName = CustomerInformation.Customer.LastName;
           CDI.Suffix = CustomerInformation.Customer.Suffix;
           CDI.Email = CustomerInformation.Customer.Email;
           CDI.PasswordHash = CustomerInformation.Customer.PasswordHash;
           CDI.Mobile = CustomerInformation.Customer.Mobile;
           CDI.Phone = CustomerInformation.Customer.Phone;
           CDI.Role = CustomerInformation.Customer.Role;
           CDI.IsLockedOut = CustomerInformation.Customer.IsLockedOut;
           CDI.DNI = CustomerInformation.Customer.DNI;
           CDI.AddressID = CustomerInformation.Addresses[0].AddressID;
           CDI.AddressLine = CustomerInformation.Addresses[0].AddressLine;
           CDI.PostalCode = CustomerInformation.Addresses[0].PostalCode;
           CDI.StateProvinveID = CustomerInformation.Addresses[0].StateProvinveID;
           CDI.City = CustomerInformation.Addresses[0].City;

            CDI.Categories = new List<int>();
            List<dtCustomer_RecoveryCategoriesResult> RCR = CustomersDataContext.dtCustomer_RecoveryCategories(IdCustomer).ToList();

            foreach (dtCustomer_RecoveryCategoriesResult RC in RCR)
            {
                CDI.Categories.Add(RC.ProductSubCategoryId);
            }

            // Retornamos la información recuperada
           return CDI;
        }

        //**************************************************************************
        //Nombre: RecoveryCustomerById()
        //Descripción: Recupera la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public dtCustomer RecoveryCustomerById(int IdCustomer)
        {
            try
            {
                // Creamos el DataContex de clientes
                dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();

                // Ejecutamos el stored procedure para recuperar la información de la base de datos.
                return CustomersDataContext.dtCustomers_RecoveryByCustomerID(IdCustomer).FirstOrDefault();
            }
            catch (Exception ex)
            {
                return null;
            }
        }


        //**************************************************************************
        //Nombre: UpdateCustomerInfo()
        //Descripción: Actualiza la información del usuario por PK = @IdCustomer
        //@params [String IdCustomer]
        //@return [CustomerInfo]
        //**************************************************************************
        public CustomerDataInfo UpdateCustomerInfo(CustomerDataInfo Customer)
        {
            try
            {
                // Creamos el DataContex de clientes
                dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();

                dtCustomers_UpdateCustomerResult CustomerUpdate = new dtCustomers_UpdateCustomerResult();
                CustomerUpdate = CustomersDataContext.dtCustomers_UpdateCustomer(Customer.CustomerID, Customer.FirstName, Customer.MiddleName, Customer.LastName, Customer.Suffix, Customer.DNI, Customer.Email, 1, Customer.Phone, Customer.Mobile, Customer.PasswordHash, Customer.Role, Customer.IsLockedOut).FirstOrDefault();

                dtAddress_UpdateResult CustomerAddressUpdate = new dtAddress_UpdateResult();
                CustomerAddressUpdate = CustomersDataContext.dtAddress_Update(Customer.AddressID, Customer.AddressLine, Customer.City, Customer.StateProvinveID, Customer.PostalCode).FirstOrDefault();

                CustomersDataContext.dtCustomerProductSubCategory_DeleteByCustomerId(Customer.CustomerID);

                dtCustomerProductSubCategory_AddNewResult PSA = new dtCustomerProductSubCategory_AddNewResult();

                foreach(int I in Customer.Categories)
                {
                    PSA = CustomersDataContext.dtCustomerProductSubCategory_AddNew(Customer.CustomerID, I).FirstOrDefault();
                }

                return RecoveryCustomerDataInfoById(Customer.CustomerID);
            }
            catch (Exception ex)
            {
                return Customer;
            }
        }

        //**************************************************************************
        //Nombre: RecoveryPassword()
        //Descripción: Recuperamos la clave de acceso por dirección de correo
        //@params [string email]
        //@return [string]
        //**************************************************************************
        public string RecoveryPassword(string email)
        {
            dtCustomer CI = new dtCustomer();
            dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
            // Ejecutamos el stored procedure para recuperar la información de la base de datos.
            CI = CustomersDataContext.dtCustomers_RecoveryCustomerByEmail(email).FirstOrDefault();
            // Si no hay registros
            if (CI == null)
            {
                return "KO";
            }
            // Si la consulta devuelve registros, enviamos la clave por correo.
            else{
                SendEmail(CI.Email, "Su clave DEOCIO", "Su clave de acceso a DEOCIO.es es: " + CI.PasswordHash);
                return "OK";
            }
        }

        //**************************************************************************
        //Nombre: SendEmail()
        //Descripción: Función que envía correos utilitzando los parametros definidos en webconfig.
        //@params [string destination, string Header, string Text]
        //@return [bool]
        //**************************************************************************
        public bool SendEmail(string destination, string Header, string Text)
        {
            try
            {
                // Leemos los parámetros definidos en el WebConfig.
                string EmailUserName = System.Configuration.ConfigurationManager.AppSettings["EmailUserName"];
                string EmailPassWord = System.Configuration.ConfigurationManager.AppSettings["EmailPassWord"];
                Int32 EmailPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["EmailPort"]);

                var message = new System.Net.Mail.MailMessage();
                message.From = new MailAddress(EmailUserName);
                message.Bcc.Add(destination);
                message.Subject = Header;
                message.Body = Text;
                message.IsBodyHtml = true;

                // Realizamos el envío  del correo.
                var client = new System.Net.Mail.SmtpClient
                {
                    Host = "smtp.gmail.com",
                    Port = EmailPort,
                    EnableSsl = true,
                    UseDefaultCredentials = false,
                    Credentials = new NetworkCredential(EmailUserName, EmailPassWord)
                };
                client.Send(message);
                // Si todo OK devolvemos true.
                return true;
            }
            catch (Exception ex)
            {
                // Si se produce algún error devolvamos false
                return false;
            }
        }

        //**************************************************************************
        //Nombre: getCustomersByCategory()
        //Descripción: Devuelve un array con información de los clientes segun la categoría pasada por parámetro
        //@params [int CategoryId]
        //@return [dtCustomer[]]
        //**************************************************************************
        public dtCustomer[] getCustomersByCategory(int CategoryId)
        {
            dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
            // Ejecutamos el stored procedure para recuperar la información de la base de datos.
            return CustomersDataContext.dtCustomers_GetCustomersByCategory(CategoryId).ToArray();
        }

        //**************************************************************************
        //Nombre: msStateProvince_All()
        //Descripción: Devuelve un listado con todas las províncias de la base de datos.
        //@params [None]
        //@return [List<msStateProvince>]
        //**************************************************************************
        public List<msStateProvince> msStateProvince_All()
        {
            msStateProvinceDataContext ProvincasDatacontext = new msStateProvinceDataContext();
            // Ejecutamos el stored procedure para recuperar la información de la base de datos.
            List<msStateProvince> Provincias = ProvincasDatacontext.msStateProvince_GetAll().ToList();
            msStateProvince BlankPR = new msStateProvince();
            BlankPR.Name = "Selecciona una província";
            BlankPR.StateProvinceID = "S";
            Provincias.Add(BlankPR);
            return Provincias;
        }

        //**************************************************************************
        //Nombre: ValidateUser()
        //Descripción: Valida si el usuario que se está validando es correcto
        //@params [string Email, string password]
        //@return [string]
        //**************************************************************************
        public string ValidateUser(string Email, string password)
        {
            if (String.IsNullOrEmpty(Email)) throw new ArgumentException("Value cannot be null or empty.", "userName");
            if (String.IsNullOrEmpty(password)) throw new ArgumentException("Value cannot be null or empty.", "password");

            try
            {
                dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
                // Ejecutamos el stored procedure para recuperar la información de la base de datos.
                dtCustomer[] Customer = CustomersDataContext.dtCustomers_RecoveryCustomerByEmail(Email.Trim()).ToArray();

                if (Customer.Length > 0)
                {
                    if (Customer[0].IsLockedOut.Equals(1))
                    {
                        // Salimos de la función
                        return "Usuario bloqueado, por favor contacte con el administrador";
                    }

                    // Si el usuario y la clave son correctos
                    if (Customer[0].PasswordHash == password.Trim())
                    {
                        // Devolvemos el rol del usuario
                        return Customer[0].Role.ToString();
                    }
                    else
                    {
                        // Salimos de la función
                        return "Usuario o clave incorrecta";
                    }
                }
                else
                {
                    // Salimos de la función
                    return "Usuario o clave incorrecta";
                }
            }

            catch (Exception ex)
            {
                // Si se ha producido algun error, salimos de la función
                return "Error de aplicación, vuelva a intentarlo pasados unos minutos...";
            }

        }

        //**************************************************************************
        //Nombre: CreateUser()
        //Descripción: Registra un nuevo usuario en la base de datos.
        //@params [string sufijo, string name, string primerapellido, string segundoapellido, string dni, string email, string password, string direccion, string cp, string poblacion, string provincia, string telefono, string movil, bool contrato, bool recibirinfo, int Role, IEnumerable<int> Categorias]
        //@return [string]
        //**************************************************************************
        public string CreateUser(string sufijo, string name, string primerapellido, string segundoapellido,
                                 string dni, string email, string password, string direccion, 
                                 string cp, string poblacion, string provincia, string telefono, string movil,
                                 bool contrato, bool recibirinfo, int Role, IEnumerable<int> Categorias)
        {
            if (String.IsNullOrEmpty(name)) throw new ArgumentException("El valor no puede ser null ni estar vacío.", "name");
            if (String.IsNullOrEmpty(password)) throw new ArgumentException("El valor no puede ser null ni estar vacío.", "password");
            if (String.IsNullOrEmpty(email)) throw new ArgumentException("El valor no puede ser null ni estar vacío.", "email");

            // Usamos una transacción, si algo falla abortamos todo el proceso.
            TransactionOptions tso = new TransactionOptions();
            tso.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;

            using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required,tso))
            {
                try
                {
                    // Definimos el DataContext
                    dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();

                    //Valiación prévia
                    dtCustomer[] ExistUser = CustomersDataContext.dtCustomers_ValidateBeforeToAdd(dni.Trim(), email.Trim()).ToArray();
                    if (ExistUser.Length > 0)
                    {
                        scope.Dispose();
                        return "El usuario introdució ya está dado de alta en la aplicación.";
                    }

                    // Ejecutamos el stored procedure para añadir al usuario.
                    int Customer = CustomersDataContext.dtCustomers_AddNewCustormer(name.Trim(), primerapellido.Trim(), segundoapellido.Trim(), sufijo.Trim(), dni.Trim(), email.Trim(), Convert.ToInt32(recibirinfo), telefono.Trim(), movil.Trim(), password.Trim(), Role);
                    // Ejecutamos el stored procedure para añadir la dirección.
                    int Address = CustomersDataContext.dtAddress_AddAddress(direccion.Trim(), poblacion.Trim(), provincia.Trim(), cp.Trim());
                    // Ejecutamos el stored procedure para añadir la relación usuario - dirección.
                    int CustomerAddres = CustomersDataContext.dtCustomerAddress_AddCustomerAddress(Customer, Address);

                    //Insertamos las categorías que el usuario ha seleccionado.
                    foreach (int Cat in Categorias)
                    {
                        dtCustomerProductSubCategory_AddNewResult status  = CustomersDataContext.dtCustomerProductSubCategory_AddNew(Customer, Cat).FirstOrDefault();
                    }
                    // Hacemos commit en la base de datos.
                    scope.Complete();


                    // Enviamos un correo electrónico a la dirección del usuario con los datos de la cuenta.
                    string text = "Su cuenta se ha creado correctamente. Usuario: " + email + " Clave: " + password;
                    string header = "Nueva cuenta en DEOCIO.es";

                    SendEmail(email, header, text);
                     
                    // Devolvemos OK
                    return "1";
                }

                catch (Exception ex)
                {
                    // Si hay algun error en la operación abortamos y salimos.
                    scope.Dispose();
                    return ex.ToString();
                }
                finally
                {
                    scope.Dispose();
                }
               
            }
                       
        }

        //**************************************************************************
        //Nombre: CustomerRecoveryAll()
        //Descripción: Devuelve la información de los clientes segun el filtro aplicado.
        //@params [string firstName,string middleName,string lastName,string dNI,string email,int? role ]
        //@return [dtCustomer_RecoveryAllResult[]]
        //**************************************************************************
        public dtCustomer_RecoveryAllResult[] CustomerRecoveryAll(string firstName,string middleName,string lastName,string dNI,string email,int? role )
        {
            dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
            dtCustomer_RecoveryAllResult[] C = CustomersDataContext.dtCustomer_RecoveryAll(firstName,middleName,lastName,dNI,email,role).ToArray();
            return C;
        }

        //**************************************************************************
        //Nombre: GetAllRoles()
        //Descripción: Devuelve todos los Roles de usuario.
        //@params [None]
        //@return [None]
        //**************************************************************************
        public msRole[] GetAllRoles()
        {
            msRolesDataContext RolesDatacontext = new msRolesDataContext();
            msRole[] Roles = RolesDatacontext.msRoles_GetAll().ToArray();
            return Roles;
        }

        //**************************************************************************
        //Nombre: UpdateStatusAccount()
        //Descripción: Actualiza el estado de la cuenta del usuario
        //@params [None]
        //@return [None]
        //**************************************************************************
        public string UpdateStatusAccount(int CustomerId, int StatusAccount)
        {
            try
            {
                dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
                int status = CustomersDataContext.dtCustomer_BlockByCustomerId(CustomerId, StatusAccount);
                if (status > 0) return "Se ha actualitzado el estado de la cuenta del usuario.";
                else return "No se ha actualizado ninguna cuenta de usuario.";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }

        }

        //**************************************************************************
        //Nombre: DeleteCustomer()
        //Descripción: Elimina un client de la base de dades
        //@params [int CustomerId]
        //@return [string]
        //**************************************************************************
        public string DeleteCustomer(int CustomerId)
        {
            try
            {
                dtCustomersDataContext CustomersDataContext = new dtCustomersDataContext();
                dtCustomer_DeleteByCustomerIDResult DelCus = CustomersDataContext.dtCustomer_DeleteByCustomerID(CustomerId).FirstOrDefault();
                if (DelCus.Column1.Equals(1))
                {
                    return "Usuario eliminado correctamente...";
                }
                else return "Error al eliminar el usuario, intentelo de nuevo...";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

    }

    
    public interface IFormsAuthenticationService
    {
        void SignIn(string userName, bool createPersistentCookie);
        void SignOut();
    }

    public class FormsAuthenticationService : IFormsAuthenticationService
    {
        public void SignIn(string userName, bool createPersistentCookie)
        {
            if (String.IsNullOrEmpty(userName)) throw new ArgumentException("Value cannot be null or empty.", "userName");

            FormsAuthentication.SetAuthCookie(userName, createPersistentCookie);
        }

        public void SignOut()
        {
            FormsAuthentication.SignOut();
        }
    }
    #endregion

    #region Validation
    public static class AccountValidation
    {
    }

    [AttributeUsage(AttributeTargets.Field | AttributeTargets.Property, AllowMultiple = false, Inherited = true)]
    public sealed class ValidatePasswordLengthAttribute : ValidationAttribute, IClientValidatable
    {
        private const string _defaultErrorMessage = "'{0}' debe tener almenos {1} caracteres de longitud.";
        private readonly int _minCharacters = Membership.Provider.MinRequiredPasswordLength;

        public ValidatePasswordLengthAttribute()
            : base(_defaultErrorMessage)
        {
        }

        public override string FormatErrorMessage(string name)
        {
            return String.Format(CultureInfo.CurrentCulture, ErrorMessageString,
                name, _minCharacters);
        }

        public override bool IsValid(object value)
        {
            string valueAsString = value as string;
            return (valueAsString != null && valueAsString.Length >= _minCharacters);
        }

        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            return new[]{
                new ModelClientValidationStringLengthRule(FormatErrorMessage(metadata.GetDisplayName()), _minCharacters, int.MaxValue)
            };
        }
    }
    #endregion

}
