﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using DEOCIO.DataModel.Products;
using System.Text;
using System.ComponentModel.DataAnnotations;
using System.Web.Security;
using System.IO;
using System.Web.Mvc;
using DEOCIO.DataModel.Orders;
using DEOCIO.DataModel.Customers;
using System.Transactions;

// Modelo de datos de las ofertas
namespace DEOCIO.Models
{
    // Modelo de datos utilizado para contener la información de cada oferta
    public class ProductModel
    {
        public int ProductID { get; set; }
        public string Name { get; set; }
        public string ProductNumber { get; set; }
        public string ProductShortDesc { get; set; }
        public string ProductDescription { get; set; }
        public decimal Price { get; set; }
        public decimal OfferPrice { get; set; }
        public int? ProductCategory { get; set; }
        public int? Status { get; set; }
        public System.Nullable<int> ProductSubcategoryID { get; set; }
        public System.DateTime SellStartDate { get; set; }
        public System.Nullable<System.DateTime> SellEndDate { get; set; }
        public System.DateTime ModifiedDate { get; set; }
        public List<string> BestOfDescription = new List<string>();
        public List<dtProductReviews> Comments = new List<dtProductReviews>();
        public int ProductBuyByUser { get; set; }
        public List<ProductPhotoAdd> ProductsPhotos = new List<ProductPhotoAdd>();
        public dtLocation Location = new dtLocation();
    }

    public class dtProductReviews
    {
        public dtProductReview dtProductReview { get; set; }
    }

    // Modelo de datos utilizado para contener la información de las imágenes de las ofertas.
    public class ProductPhotoAdd
    { 
        public string ThumbnailPhotoFileName { get; set; }
        public string ThumbnailPhotoType { get; set; }
        public string LargePhotoFileName { get; set; } 
        public string LargePhotoType { get; set; }
        public System.Data.Linq.Binary LargeImage64Bits { get; set; }
        public System.Data.Linq.Binary Thumbnail64Bits { get; set; }
        public string primaria  {get; set; }
    }

    public class CommandInfo
    {
        public System.DateTime OrderDate { get; set; }
        public byte CommandStatus { get; set; }
        public int? Status { get; set; }
        public string SalesOrderNumber { get; set; }
        public int BillToAddressID { get; set; }
        public int ShipToAddressID { get; set; }
        public decimal SubTotal { get; set; }
        public decimal TaxAmt { get; set; }
        public decimal Freight { get; set; }
        public decimal TotalDue { get; set; }
        public string Comment { get; set; }
        public int SalesOrderID { get; set; }
        public int SalesOrderDetailID { get; set; }
        public string CarrierTrackingNumber { get; set; }
        public short OrderQty { get; set; }
        public decimal UnitPrice { get; set; }
        public decimal UnitPriceOffer { get; set; }
        public decimal LineTotal { get; set; }
        public int CustomerID { get; set; }
        public string FirstName { get; set; }
        public string MiddleName { get; set; }
        public string LastName { get; set; }
        public string Suffix { get; set; }
        public string DNI { get; set; }
        public string Email { get; set; }
        public int EmailPromotion { get; set; }
        public string Phone { get; set; }
        public string Mobile { get; set; }
        public string PasswordHash { get; set; }
        public int Role { get; set; }
        public int IsApproved { get; set; }
        public System.Nullable<int> IsLockedOut { get; set; }
        public System.DateTime ModifiedDate { get; set; }
        public int AddressID { get; set; }
        public string AddressLine { get; set; }
        public string City { get; set; }
        public string StateProvinveID { get; set; }
        public string PostalCode { get; set; }
        public int? Categories { get; set; }
        public int ProductID { get; set; }
        public string Name { get; set; }
        public string ProductNumber { get; set; }
        public string ProductShortDesc { get; set; }
        public string ProductDescription { get; set; }
        public decimal Price { get; set; }
        public decimal OfferPrice { get; set; }
        public int? ProductCategory { get; set; }
        public System.Nullable<int> ProductSubcategoryID { get; set; }
        public System.DateTime SellStartDate { get; set; }
        public System.Nullable<System.DateTime> SellEndDate { get; set; }
    }

    // Modelo de datos utilizado para contener información de la orden de compra
    public class ConfirmationBuy
    {
        public ProductModel ProductBuy { get; set; }
        public dtSalesOrderDetail OrderProductDetail { get; set; }
        public dtSalesOrderHeader OrderHeader { get; set; }
        public dtCustomer Customer { get; set; }
    }

    // Modelo de datos utilizado para contener la información de los comentarios de productos.
    public class Comments
    {
        [Required]
        public string txtComment { get; set; }

        [Required]
        public string txtRating { get; set; }

        [Required]
        public int ProductId { get; set; }
    }

    // Modelo de datos utilizado recuperar cierta información de las órdenes de compra.
    public class OrderInfo
    {
        public string OrderId { get; set; }
        public string OrderDate { get; set; }
        public string ProductId { get; set; }
        public string ProductDesc { get; set; }
        public string ProductOfferPrice { get; set; }
        public string CustomerName { get; set; }
        public string Cust_DNI { get; set; }
        public string Cust_Phone { get; set; }
        public string Cust_Email { get; set; }
    }

    // Se definen que métodos contendrá el servicio.
    public interface ProductService
    {
        List<ProductModel> getLastProducts();
        List<ProductModel> getRandomProducts();
        ProductModel getProductById(int ProductId, int Customer);
        dtProductPhoto_RecoveryByIdResult[] ProductPhoto_RecoveryById(int Id);
        dtProductPhoto[] ProductPhoto_RecoveryByPhotoId(int Id);
        List<ProductModel> getProductsByQuery(string txtPoblacion, int? txtDistancia, decimal? PrecioDesde, decimal? PrecioHasta, DateTime Desde, DateTime Hasta, string Ordenarpor, int? Category);
        List<int> getmaxProductBuy(Int32 IdProduct);
        string BuyOffer(string NumProduct, string ProductID, string SelAddress, CustomerInfo CustomerInf);
        string addComment(string Comment, int ProductId, CustomerInfo CI, string txtPuntuacion);
        ConfirmationBuy getCommandInfo(int OrderId);
        List<dtSalesOrder_RecoveryByUserResult> MisCompras(CustomerInfo CI);
        List<ProductModel> getAllProducts(string Name, string ProductNumber, string ShortDesc, int? ProductCategory);
        dtProductSubCategory[] ProductSubCategory_GetAll();
        string ProductSubCategory_Delete(int ProductSubcategoryId);
        string ProductSubCategory_Update(int ProductSubcategoryId, string Name);
        string ProductSubCategory_Insert(string Name);
        dtProductLocation_RecoveryByProductIdResult GetProductLocation(int ProductID);
        CommandInfo[] GetAllOrders(int? SalesOrderID, DateTime? OrderDate, string ProductShortDesc, int? ProductSubcategoryID, string ProductNumber, string Email, string DNI);
        string UpdateProductStatus(int ProductID, int Status);
        CommandInfo[] GetOrderById(int OrderId);
        ProductModel UpdateProducts(ProductModel PM);
    }
    
    public class ProductModels : ProductService
    {

        //**************************************************************************
        //Nombre: getLastProducts()
        //Descripción: Obtiene la lista de ofertas vigentes sin filtro (máximo 12)
        //@params [None]
        //@return [List<ProductModel>]
        //**************************************************************************
        public List<ProductModel> getLastProducts()
        {
            // Creamos el contexto de datos.
            ProductsDataContext DataContextProduct = new ProductsDataContext();

            // Ejecutamos el stored procedure para obtener los datos (solo recuperamos los ID's para optimizar la consulta.)
            DEOCIO.DataModel.Products.dtProducts_RecoveryLastResult[] ProductsId = DataContextProduct.dtProducts_RecoveryLast().ToArray();

            List<ProductModel> resultProducts = new List<ProductModel>();
            StringBuilder pIDs = new StringBuilder("");

            foreach (DEOCIO.DataModel.Products.dtProducts_RecoveryLastResult PKId in ProductsId)
            {

                if (pIDs.Length.Equals(0))
                {
                    pIDs.Append(PKId.ProductID.ToString());
                }
                else
                {
                    pIDs.Append(",");
                    pIDs.Append(PKId.ProductID.ToString());
                }
            }

            // Obtenemos la información completa de todas las ofertas con ID = @pIDS
            dtProducts_RecoveryByIdResult[] Products = DataContextProduct.dtProducts_RecoveryById(pIDs.ToString()).ToArray();
            DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(pIDs.ToString()).ToArray();

            foreach (dtProducts_RecoveryByIdResult pr in Products)
            {
                ProductModel PM = new ProductModel();
                PM.ProductID = pr.ProductID;
                PM.ProductNumber = pr.ProductNumber;
                PM.Name = pr.Name;
                PM.ProductShortDesc = pr.ProductShortDesc;
                PM.ProductDescription = pr.ProductDescription;
                PM.Price = pr.Price;
                PM.OfferPrice = pr.OfferPrice;
                PM.SellStartDate = pr.SellStartDate;
                PM.SellEndDate = pr.SellEndDate;
                PM.ModifiedDate = pr.ModifiedDate;

                var numsPlusOne = from n in BestOfDescriptions where n.ProductID == pr.ProductID select n;

                foreach (DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
                {
                    PM.BestOfDescription.Add(k.Description.ToString());
                }

                // Devolvemos los resultados.
                resultProducts.Add(PM);

            }
            return resultProducts;
        }

        //**************************************************************************
        //Nombre: getProductsByCategory()
        //Descripción: Devuelve todas las ofertas vinculadas a una categoría
        //@params [int CategoryId]
        //@return [dtProduct[]]
        //**************************************************************************
        public dtProduct[] getProductsByCategory(int CategoryId)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            return DataContextProduct.dtProducts_GetProductsByCategory(CategoryId).ToArray();
        }


        //**************************************************************************
        //Nombre: getRandomProducts()
        //Descripción: Obtiene una lista de seis ofertas aleatórias para mostrar en la parte inferior de la página.
        //@params [None]
        //@return [List<ProductModel>]
        //**************************************************************************
        public List<ProductModel> getRandomProducts()
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            DEOCIO.DataModel.Products.dtProduct_RecoveryRandomResult[] ProductsId = DataContextProduct.dtProduct_RecoveryRandom().ToArray();

            List<ProductModel> resultProducts = new List<ProductModel>();

            StringBuilder pIDs = new StringBuilder("");


            foreach (DEOCIO.DataModel.Products.dtProduct_RecoveryRandomResult PKId in ProductsId)
            {

                if (pIDs.Length.Equals(0))
                {
                    pIDs.Append(PKId.ProductID.ToString());
                }
                else
                {
                    pIDs.Append(",");
                    pIDs.Append(PKId.ProductID.ToString());
                }
            }

            dtProducts_RecoveryByIdResult[] Products = DataContextProduct.dtProducts_RecoveryById(pIDs.ToString()).ToArray();
            DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(pIDs.ToString()).ToArray();

            foreach (dtProducts_RecoveryByIdResult pr in Products)
            {
                ProductModel PM = new ProductModel();
                PM.ProductID = pr.ProductID;
                PM.ProductNumber = pr.ProductNumber;
                PM.Name = pr.Name;
                PM.ProductShortDesc = pr.ProductShortDesc;
                PM.ProductDescription = pr.ProductDescription;
                PM.Price = pr.Price;
                PM.OfferPrice = pr.OfferPrice;
                PM.SellStartDate = pr.SellStartDate;
                PM.SellEndDate = pr.SellEndDate;
                PM.ModifiedDate = pr.ModifiedDate;

                var numsPlusOne = from n in BestOfDescriptions where n.ProductID == pr.ProductID select n;

                foreach (DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
                {
                    PM.BestOfDescription.Add(k.Description.ToString());
                }

                resultProducts.Add(PM);

            }
            return resultProducts;
        }

        //**************************************************************************
        //Nombre: getCommandInfo()
        //Descripción: Obtiene información de la orden de compra
        //@params [int HeaderId]
        //@return [ConfirmationBuy]
        //**************************************************************************
        public ConfirmationBuy getCommandInfo(int HeaderId)
        {
            ConfirmationBuy CB = new ConfirmationBuy();
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            OrdersDataContext DataContextOrder = new OrdersDataContext();
            dtCustomersDataContext DataContextCustomer = new dtCustomersDataContext();

            CB.OrderHeader = DataContextOrder.dtSalesOrderHeader_RecoveryById(HeaderId).FirstOrDefault();
            CB.OrderProductDetail = DataContextOrder.dtSalesOrderDetail_RecoveryById(CB.OrderHeader.SalesOrderID).FirstOrDefault();
            CB.Customer = DataContextCustomer.dtCustomers_RecoveryByCustomerID(CB.OrderHeader.CustomerID).FirstOrDefault();
            CB.ProductBuy = getProductById(CB.OrderProductDetail.ProductID, CB.Customer.CustomerID);

            return CB;
        }

        //**************************************************************************
        //Nombre: GetAllOrders()
        //Descripción: Obtiene un array de todas las compras realizadas
        //@params [None]
        //@return [None]
        //**************************************************************************
        public CommandInfo[] GetAllOrders(int? SalesOrderID, DateTime? OrderDate, string ProductShortDesc,  int? ProductSubcategoryID, string ProductNumber, string Email, string DNI)
        {

            List<CommandInfo> CBS = new List<CommandInfo>();
            OrdersDataContext DataContextOrder = new OrdersDataContext();
            dtSalesOrderHeader_GetAllResult[] Ordenes = DataContextOrder.dtSalesOrderHeader_GetAll(SalesOrderID, OrderDate, ProductShortDesc, ProductSubcategoryID, ProductNumber, Email, DNI).ToArray();

            foreach (dtSalesOrderHeader_GetAllResult Orden in Ordenes)
            {

                CommandInfo CB = new CommandInfo();
                CB.SalesOrderID = Orden.SalesOrderID;
                CB.OrderDate = Orden.OrderDate;
                CB.CommandStatus = Orden.CommandStatus;
                CB.SalesOrderNumber = Orden.SalesOrderNumber;
                CB.CustomerID = Orden.CustomerID;
                CB.BillToAddressID = Orden.BillToAddressID;
                CB.ShipToAddressID = Orden.ShipToAddressID;
                CB.SubTotal = Orden.SubTotal;
                CB.TaxAmt = Orden.TaxAmt;
                CB.Freight = Orden.Freight;
                CB.TotalDue = Orden.TotalDue;
                CB.Comment = Orden.Comment;
                CB.SalesOrderDetailID = Orden.SalesOrderDetailID;
                CB.CarrierTrackingNumber = Orden.CarrierTrackingNumber;
                CB.OrderQty = Orden.OrderQty;
                CB.ProductID = Orden.ProductID;
                CB.UnitPrice = Orden.UnitPrice;
                CB.UnitPriceOffer = Orden.UnitPriceOffer;
                CB.LineTotal = Orden.LineTotal;
                CB.FirstName = Orden.FirstName;
                CB.MiddleName = Orden.MiddleName;
                CB.DNI = Orden.DNI;
                CB.Suffix = Orden.Suffix;
                CB.Phone = Orden.Phone;
                CB.Mobile = Orden.Mobile;
                CB.Email = Orden.Email;
                CB.EmailPromotion = Orden.EmailPromotion;
                CB.Name = Orden.Name;
                CB.ProductNumber = Orden.ProductNumber;
                CB.ProductShortDesc = Orden.ProductShortDesc;
                CB.ProductDescription = Orden.ProductDescription;
                CB.Price = Orden.Price;
                CB.OfferPrice = Orden.OfferPrice;
                CB.SellStartDate = Orden.SellStartDate;
                CB.SellEndDate = Orden.SellEndDate;
                CB.Status = Orden.Status;
            
                CBS.Add(CB);
            }

            return CBS.ToArray();
        }

        //**************************************************************************
        //Nombre: UpdateProducts()
        //Descripción: Actualiza la información de una oferta en la base de datos.
        //@params [ProductModel PM]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel UpdateProducts(ProductModel PM)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();

            DataContextProduct.dtProducts_Update(PM.ProductID, PM.Name, PM.ProductNumber, PM.ProductShortDesc, PM.ProductDescription, PM.Price, PM.Status, PM.OfferPrice, PM.ProductSubcategoryID, PM.SellStartDate, PM.SellEndDate);

            return GetProductOnlyById(PM.ProductID);
        }


        //**************************************************************************
        //Nombre: GetOrderById()
        //Descripción: Obtiene un array de la orden por PK
        //@params [int OrderId]
        //@return [None]
        //**************************************************************************
        public CommandInfo[] GetOrderById(int OrderId)
        {
            List<CommandInfo> CBS = new List<CommandInfo>();
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            OrdersDataContext DataContextOrder = new OrdersDataContext();
            dtCustomersDataContext DataContextCustomer = new dtCustomersDataContext();

            dtSalesOrderHeader[] Ordenes = DataContextOrder.dtSalesOrderHeader_GetByPk(OrderId).ToArray();
            foreach (dtSalesOrderHeader Orden in Ordenes)
            {

                dtSalesOrderDetail Detail = DataContextOrder.dtSalesOrderDetail_RecoveryById(Orden.SalesOrderID).FirstOrDefault();
                dtCustomer Customer = DataContextCustomer.dtCustomers_RecoveryByCustomerID(Orden.CustomerID).FirstOrDefault();
                ProductModel Product = getProductById(Detail.ProductID, Customer.CustomerID);


                CommandInfo CB = new CommandInfo();
                CB.SalesOrderID = Orden.SalesOrderID;
                CB.OrderDate = Orden.OrderDate;
                CB.Status = Orden.Status;
                CB.SalesOrderNumber = Orden.SalesOrderNumber;
                CB.CustomerID = Orden.CustomerID;
                CB.BillToAddressID = Orden.BillToAddressID;
                CB.ShipToAddressID = Orden.ShipToAddressID;
                CB.SubTotal = Orden.SubTotal;
                CB.TaxAmt = Orden.TaxAmt;
                CB.Freight = Orden.Freight;
                CB.TotalDue = Orden.TotalDue;
                CB.Comment = Orden.Comment;
                CB.SalesOrderDetailID = Detail.SalesOrderDetailID;
                CB.CarrierTrackingNumber = Detail.CarrierTrackingNumber;
                CB.OrderQty = Detail.OrderQty;
                CB.ProductID = Detail.ProductID;
                CB.UnitPrice = Detail.UnitPrice;
                CB.UnitPriceOffer = Detail.UnitPriceOffer;
                CB.LineTotal = Detail.LineTotal;
                CB.FirstName = Customer.FirstName;
                CB.MiddleName = Customer.MiddleName;
                CB.LastName = Customer.LastName;
                CB.DNI = Customer.DNI;
                CB.Suffix = Customer.Suffix;
                CB.Phone = Customer.Phone;
                CB.Mobile = Customer.Mobile;
                CB.Email = Customer.Email;
                CB.EmailPromotion = Customer.EmailPromotion;
                CB.Name = Product.Name;
                CB.ProductNumber = Product.ProductNumber;
                CB.ProductShortDesc = Product.ProductShortDesc;
                CB.ProductDescription = Product.ProductDescription;
                CB.Price = Product.Price;
                CB.OfferPrice = Product.OfferPrice;
                CB.SellStartDate = Product.SellStartDate;
                CB.SellEndDate = Product.SellEndDate;
                CB.Categories = Product.ProductSubcategoryID;
                

                CBS.Add(CB);
            }

            return CBS.ToArray();
        }

        //**************************************************************************
        //Nombre: addComment()
        //Descripción: Añade un nuevo comentario.
        //@params [string Comment, int ProductId, CustomerInfo CI, string txtPuntuacion]
        //@return [string]
        //**************************************************************************
        public string addComment(string Comment, int ProductId, CustomerInfo CI, string txtPuntuacion)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            DataContextProduct.dtProductReview_InsertReview(ProductId, CI.Customer.FirstName, CI.Customer.Email, Convert.ToInt32(txtPuntuacion), Comment);
            return "";
        }

        //**************************************************************************
        //Nombre: getProductById()
        //Descripción: Recupera la información de un producto por ProductId y CustomerID
        //@params [int ProductId, int Customer]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel getProductById(int ProductId, int Customer)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            dtProducts_RecoveryByIdResult Products = DataContextProduct.dtProducts_RecoveryById(ProductId.ToString()).FirstOrDefault();
            dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(ProductId.ToString()).ToArray();
            dtProductReview[] Comments = DataContextProduct.dtProductReview_RecoveryByProductId(ProductId).ToArray();

            ProductModel PM = new ProductModel();
            PM.ProductID = Products.ProductID;
            PM.ProductNumber = Products.ProductNumber;
            PM.Name = Products.Name;
            PM.ProductShortDesc = Products.ProductShortDesc;
            PM.ProductDescription = Products.ProductDescription;
            PM.Price = Products.Price;
            PM.OfferPrice = Products.OfferPrice;
            PM.SellStartDate = Products.SellStartDate;
            PM.SellEndDate = Products.SellEndDate;
            PM.ModifiedDate = Products.ModifiedDate;
            PM.ProductSubcategoryID = Products.ProductSubcategoryID;

            PM.ProductBuyByUser = Convert.ToInt32(DataContextProduct.dtOrders_VerifyCutomerBuyProduct(Customer, PM.ProductID).FirstOrDefault().valor);

            var numsPlusOne = from n in BestOfDescriptions where n.ProductID == Products.ProductID select n;

            foreach (dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
            {
                PM.BestOfDescription.Add(k.Description.ToString());
            }

            foreach (dtProductReview PR in Comments)
            {
                dtProductReviews PPR = new dtProductReviews();
                PPR.dtProductReview = PR;
                PM.Comments.Add(PPR);
            }

            return PM;
        }

        //**************************************************************************
        //Nombre: ProductPhoto_RecoveryById()
        //Descripción: Recupera las imagenes vinculadas a una oferta
        //@params [int Id]
        //@return [dtProductPhoto_RecoveryByIdResult[]]
        //**************************************************************************
        public dtProductPhoto_RecoveryByIdResult[] ProductPhoto_RecoveryById(int Id)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            dtProductPhoto_RecoveryByIdResult[] ImageBinary = DataContextProduct.dtProductPhoto_RecoveryById(Id).ToArray();
            return ImageBinary;
        }

        //**************************************************************************
        //Nombre: ProductPhoto_RecoveryByPhotoId()
        //Descripción: Recupera la foto por Id de foto.
        //@params [int Id]
        //@return [dtProductPhoto[]]
        //**************************************************************************
        public dtProductPhoto[] ProductPhoto_RecoveryByPhotoId(int Id)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            dtProductPhoto[] ImageBinary = DataContextProduct.dtProductPhoto_RecoveryByPhotoId(Id).ToArray();
            return ImageBinary;
        }

        //**************************************************************************
        //Nombre: getProductsByQuery()
        //Descripción: Devuelve una lista con la información de las ofertas según el filtro aplicado
        //@params [string txtPoblacion, int? txtDistancia, decimal? PrecioDesde, decimal? PrecioHasta, DateTime Desde, DateTime Hasta, string Ordenarpor, int? Category]
        //@return [List<ProductModel>]
        //**************************************************************************
        public List<ProductModel> getProductsByQuery(string txtPoblacion, int? txtDistancia, decimal? PrecioDesde, decimal? PrecioHasta, DateTime Desde, DateTime Hasta, string Ordenarpor, int? Category)
        {
            // Definimos el contexto de datos 
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            DEOCIO.DataModel.Products.dtProducts_RecoveryQueryResult[] ProductsId = DataContextProduct.dtProducts_RecoveryQuery(txtPoblacion, txtDistancia, PrecioDesde, PrecioHasta, Desde, Hasta, Ordenarpor, Category).ToArray();
            
            List<ProductModel> resultProducts = new List<ProductModel>();
            StringBuilder pIDs = new StringBuilder("");

            foreach (DEOCIO.DataModel.Products.dtProducts_RecoveryQueryResult PKId in ProductsId)
            {
                if (pIDs.Length.Equals(0))
                {
                    pIDs.Append(PKId.ProductID.ToString());
                }
                else
                {
                    pIDs.Append(",");
                    pIDs.Append(PKId.ProductID.ToString());
                }
            }

            dtProducts_RecoveryByIdResult[] Products = DataContextProduct.dtProducts_RecoveryById(pIDs.ToString()).ToArray();
            DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(pIDs.ToString()).ToArray();

            foreach (dtProducts_RecoveryByIdResult pr in Products)
            {
                ProductModel PM = new ProductModel();
                PM.ProductID = pr.ProductID;
                PM.ProductNumber = pr.ProductNumber;
                PM.Name = pr.Name;
                PM.ProductShortDesc = pr.ProductShortDesc;
                PM.ProductDescription = pr.ProductDescription;
                PM.Price = pr.Price;
                PM.OfferPrice = pr.OfferPrice;
                PM.SellStartDate = pr.SellStartDate;
                PM.SellEndDate = pr.SellEndDate;
                PM.ModifiedDate = pr.ModifiedDate;

                var numsPlusOne = from n in BestOfDescriptions where n.ProductID == pr.ProductID select n;

                foreach (DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
                {
                    PM.BestOfDescription.Add(k.Description.ToString());
                }

                dtProductReview[] Comments = DataContextProduct.dtProductReview_RecoveryByProductId(PM.ProductID).ToArray();

                foreach (dtProductReview PR in Comments)
                {
                    dtProductReviews PPR = new dtProductReviews();
                    PPR.dtProductReview = PR;
                    PM.Comments.Add(PPR);
                }

                resultProducts.Add(PM);
            }

            // Definimos la ordenación de las ofertas devueltas.
            List<ProductModel> ProductOrdened = new List<ProductModel>();

            if (Ordenarpor == "OfferPrice")
            {
                var tempo = resultProducts.OrderBy(x => x.OfferPrice);
                foreach (ProductModel Pmt in tempo)
                {
                    ProductOrdened.Add(Pmt);
                }
                return ProductOrdened;
            }
            else if (Ordenarpor == "Votes")
            {
                var tempo = resultProducts.OrderByDescending(x => x.Comments.Count());
                foreach (ProductModel Pmt in tempo)
                {
                    ProductOrdened.Add(Pmt);
                }
                return ProductOrdened;
            }
            else{
                return resultProducts;
                }
        }

        //**************************************************************************
        //Nombre: getAllProducts()
        //Descripción: Devuelve los productos según el filtro aplicado (DEOCIOAPP)
        //@params [string Name, string ProductNumber, string ShortDesc, int? ProductCategory]
        //@return [List<ProductModel>]
        //**************************************************************************
        public List<ProductModel> getAllProducts(string Name, string ProductNumber, string ShortDesc, int? ProductCategory)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            DEOCIO.DataModel.Products.dtProducts_RecoveryAllResult [] ProductsId = DataContextProduct.dtProducts_RecoveryAll(Name, ProductNumber, ShortDesc, ProductCategory).ToArray();

            List<ProductModel> resultProducts = new List<ProductModel>();


            StringBuilder pIDs = new StringBuilder("");

            foreach (DEOCIO.DataModel.Products.dtProducts_RecoveryAllResult PKId in ProductsId)
            {

                if (pIDs.Length.Equals(0))
                {
                    pIDs.Append(PKId.ProductID.ToString());
                }
                else
                {
                    pIDs.Append(",");
                    pIDs.Append(PKId.ProductID.ToString());
                }
            }

            dtProducts_RecoveryByIdResult[] Products = DataContextProduct.dtProducts_RecoveryById(pIDs.ToString()).ToArray();
            DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(pIDs.ToString()).ToArray();

            foreach (dtProducts_RecoveryByIdResult pr in Products)
            {
                ProductModel PM = new ProductModel();
                PM.ProductID = pr.ProductID;
                PM.ProductNumber = pr.ProductNumber;
                PM.Name = pr.Name;
                PM.ProductShortDesc = pr.ProductShortDesc;
                PM.ProductDescription = pr.ProductDescription;
                PM.Price = pr.Price;
                PM.OfferPrice = pr.OfferPrice;
                PM.SellStartDate = pr.SellStartDate;
                PM.SellEndDate = pr.SellEndDate;
                PM.ModifiedDate = pr.ModifiedDate;
                PM.Status = pr.Status;

                var numsPlusOne = from n in BestOfDescriptions where n.ProductID == pr.ProductID select n;

                foreach (DEOCIO.DataModel.Products.dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
                {
                    PM.BestOfDescription.Add(k.Description.ToString());
                }

                resultProducts.Add(PM);

            }
           
                return resultProducts;
        }

        //**************************************************************************
        //Nombre: AddNewProduct()
        //Descripción: Añade una nueva oferta en la base de datos.
        //@params [ProductModel PM]
        //@return [bool]
        //**************************************************************************
        public bool AddNewProduct(ProductModel PM)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            int P_ID = DataContextProduct.dtProducts_AddNew(PM.Name, PM.ProductNumber, PM.ProductShortDesc, PM.ProductDescription, PM.Price, PM.OfferPrice, PM.ProductSubcategoryID, PM.SellStartDate, PM.SellEndDate);
            if (P_ID > 0)
            {
                foreach (ProductPhotoAdd dPPH in PM.ProductsPhotos)
                {
                    int PH_ID = DataContextProduct.dtProductPhoto_AddNew(dPPH.Thumbnail64Bits, dPPH.ThumbnailPhotoFileName, dPPH.ThumbnailPhotoType, dPPH.LargeImage64Bits, dPPH.LargePhotoType, dPPH.LargePhotoType);
                    if (PH_ID > 0)
                    {
                        if (dPPH.primaria == "P")
                        {
                            DataContextProduct.dtProductProductPhoto_AddNew(P_ID, PH_ID, 0);
                        }
                        else
                        {
                            DataContextProduct.dtProductProductPhoto_AddNew(P_ID, PH_ID, 1);
                        }
                    }
                }

                foreach (string BOF in PM.BestOfDescription)
                {
                    int BO_ID = DataContextProduct.dtBestOfProduct_AddNew(BOF);
                    if (BO_ID > 0)
                    {
                        DataContextProduct.dtProductBestOfProduct_AddNew(P_ID, BO_ID);
                    }
                }

                int LO_ID = DataContextProduct.dtLocation_AddNew(PM.Location.AddressLine1, PM.Location.AddressLine2, PM.Location.City, PM.Location.StateProvinceID, PM.Location.PostalCode);
                DataContextProduct.dtProductLocation_AddNew(LO_ID, P_ID);
                return true;
            }
            else
            {
                return false;
            }

        }

        //**************************************************************************
        //Nombre: GetProductOnlyById()
        //Descripción: Recupera una oferta solo por PK (DEOCIO APP) 
        //@params [int ProductID]
        //@return [ProductModel]
        //**************************************************************************
        public ProductModel GetProductOnlyById(int ProductID)
        {
           ProductModel PM = new ProductModel();
            
            try{
                ProductsDataContext DataContextProduct = new ProductsDataContext(); 
                dtSalesOrderDetail_RecoveryByProductIdResult ExistSalesOrders = DataContextProduct.dtSalesOrderDetail_RecoveryByProductId(ProductID).SingleOrDefault();

               
                dtProducts_RecoveryByIdResult Products = DataContextProduct.dtProducts_RecoveryById(ProductID.ToString()).FirstOrDefault();
                dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(ProductID.ToString()).ToArray();
                dtProductReview[] Comments = DataContextProduct.dtProductReview_RecoveryByProductId(ProductID).ToArray();

            
                PM.ProductID = Products.ProductID;
                PM.ProductNumber = Products.ProductNumber;
                PM.Name = Products.Name;
                PM.ProductShortDesc = Products.ProductShortDesc;
                PM.ProductDescription = Products.ProductDescription;
                PM.Price = Products.Price;
                PM.OfferPrice = Products.OfferPrice;
                PM.SellStartDate = Products.SellStartDate;
                PM.SellEndDate = Products.SellEndDate;
                PM.ModifiedDate = Products.ModifiedDate;
                PM.ProductCategory = Products.ProductSubcategoryID;
                PM.Status = Products.Status;

                var numsPlusOne = from n in BestOfDescriptions where n.ProductID == Products.ProductID select n;

                foreach (dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
                {
                    PM.BestOfDescription.Add(k.Description);
                }
                
                dtProductLocation_RecoveryByProductIdResult Location = DataContextProduct.dtProductLocation_RecoveryByProductId(PM.ProductID).FirstOrDefault();

                PM.Location.AddressLine1 = Location.AddressLine1;
                PM.Location.AddressLine2 = Location.AddressLine2;
                PM.Location.City = Location.City;
                PM.Location.PostalCode = Location.PostalCode;
                PM.Location.StateProvinceID = Convert.ToInt32(Location.StateProvinceCode);

                dtProductProductPhoto[] PPP = DataContextProduct.dtProductProductPhoto_RecoveryByProductId(PM.ProductID).ToArray();

                foreach (dtProductProductPhoto Pht in PPP)
                {
                    dtProductPhoto dtPH = DataContextProduct.dtProductPhoto_RecoveryByPhotoId(Pht.ProductPhotoID).FirstOrDefault();
                    ProductPhotoAdd PA = new ProductPhotoAdd();
                    PA.LargePhotoFileName = dtPH.LargePhotoFileName;
                    PA.LargePhotoType = dtPH.LargePhotoType;

                    PA.ThumbnailPhotoFileName = dtPH.ThumbnailPhotoFileName;
                    PA.ThumbnailPhotoType = dtPH.ThumbnailPhotoType;

                    PM.ProductsPhotos.Add(PA);
                }

                return PM;
            }
            catch(Exception ex)
            {
                return PM;
            }

        }

        //**************************************************************************
        //Nombre: DeleteProduct()
        //Descripción: Elimina un producto y sus dependencias de la base de datos (siempre que no haya una compra de este producto)
        //@params [int ProductID]
        //@return [string]
        //**************************************************************************
        public string DeleteProduct(int ProductID)
        {
            try
            {
                // Definimos el contexto de datos
                ProductsDataContext DataContextProduct = new ProductsDataContext();
                // Verificamos si existe alguna orden que contenga este producto
                dtSalesOrderDetail_RecoveryByProductIdResult ExistSalesOrders = DataContextProduct.dtSalesOrderDetail_RecoveryByProductId(ProductID).SingleOrDefault();
                // Si es así abortamos.
                if (ExistSalesOrders.Column1 > 0)
                {
                    return "No se pueden eliminar ofertas con pedidos realizados";
                }
                else
                {
                    // Recuperamos la información del producto
                    dtProducts_RecoveryByIdResult Products = DataContextProduct.dtProducts_RecoveryById(ProductID.ToString()).FirstOrDefault();
                    // Recuperamos las descripciones vinculadas de lo mejor de la oferta.
                    dtProducts_RecoberyBestOfProductByIdResult[] BestOfDescriptions = DataContextProduct.dtProducts_RecoberyBestOfProductById(ProductID.ToString()).ToArray();
                    // Recuperamos los comentarios vinculados a la oferta.
                    dtProductReview[] Comments = DataContextProduct.dtProductReview_RecoveryByProductId(ProductID).ToArray();

                    ProductModel PM = new ProductModel();
                    PM.ProductID = Products.ProductID;
                    PM.ProductNumber = Products.ProductNumber;
                    PM.Name = Products.Name;
                    PM.ProductShortDesc = Products.ProductShortDesc;
                    PM.ProductDescription = Products.ProductDescription;
                    PM.Price = Products.Price;
                    PM.OfferPrice = Products.OfferPrice;
                    PM.SellStartDate = Products.SellStartDate;
                    PM.SellEndDate = Products.SellEndDate;
                    PM.ModifiedDate = Products.ModifiedDate;

                    var numsPlusOne = from n in BestOfDescriptions where n.ProductID == Products.ProductID select n;

                    // Eliminamos la relación oferta -- descripción lo mejor de la oferts
                    DataContextProduct.dtProductBestOfProduct_DeleteById(PM.ProductID);

                    // Para cada descripción 
                    foreach (dtProducts_RecoberyBestOfProductByIdResult k in numsPlusOne)
                    {
                        // Eliminamos la descripción
                        DataContextProduct.dtBestOfProduct_DeleteById(k.BestOfProductId);
                    }

                    // Para cada comentario
                    foreach (dtProductReview PR in Comments)
                    {
                        // Eliminamos comentario
                        DataContextProduct.dtProductReview_DeleteById(PR.ProductID);
                    }

                    // Recuperamos las fotos vinculadas a la oferta
                    dtProductProductPhoto[] PPP = DataContextProduct.dtProductProductPhoto_RecoveryByProductId(PM.ProductID).ToArray();

                    // Eliminamos las relaciones oferta - fotos.
                    DataContextProduct.dtProductProductPhoto_DeleteByID(PM.ProductID);

                    // Para cada foto
                    foreach (dtProductProductPhoto Pht in PPP)
                    {
                        // Eliminamos foto
                        DataContextProduct.dtProductPhoto_DeleteById(Pht.ProductPhotoID);
                    }

                    // Eliminamos la localización de la oferta
                    DataContextProduct.dtProductLocation_DeleteByProductId(PM.ProductID);

                    // Finalmente eliminamos la oferta.
                    DataContextProduct.dtProduct_DeleteById(PM.ProductID);

                    // Devolvemos mensaje
                    return "Oferta eliminada correctamente.";
                }
            }catch(Exception E)
            {
                // Si se ha producido algún error devolvemos mensaje error.
                return E.Message;
            }

        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Insert()
        //Descripción: Insertar una categoría en la base de datos
        //@params [string Name]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Insert(string Name)
        {
            try
            {
                ProductsDataContext DataContextProduct = new ProductsDataContext();
                dtProductSubCategory_InsertNewResult result = DataContextProduct.dtProductSubCategory_InsertNew(Name).FirstOrDefault();
                if (result.REGISTROS > 0) return "Categoría insertada correctamente";
                else return "No ha sido posible insertar la categoría, vuelva a intentarlo";
            }
            catch(Exception ex)
            {
                return ex.Message;
            }

        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Update()
        //Descripción: Actualiza la descripción deuna categoría
        //@params [int ProductSubcategoryId, string Name]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Update(int ProductSubcategoryId, string Name)
        {
            try
            {
                ProductsDataContext DataContextProduct = new ProductsDataContext();
                dtProductSubCategory_UpdateResult result = DataContextProduct.dtProductSubCategory_Update(ProductSubcategoryId, Name).FirstOrDefault();
                if (result.REGISTROS > 0) return "Categoría modificada correctamente";
                else return "No ha sido posible modificar la categoría, vuelva a intentarlo";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_GetAll()
        //Descripción: Devuelve un array con las categorías definidas en la base de datos
        //@params [None]
        //@return [dtProductSubCategory[]]
        //**************************************************************************
        public dtProductSubCategory[] ProductSubCategory_GetAll()
        {
            try
            {
                ProductsDataContext DataContextProduct = new ProductsDataContext();
                dtProductSubCategory[] DT = DataContextProduct.dtProductSubCategory_GetAll().ToArray();
                return DT;
            }
            catch(Exception ex)
            {
                return null;
            }
            
        }

        //**************************************************************************
        //Nombre: ProductSubCategory_Delete()
        //Descripción: Elimina una categoría de la base de datos
        //@params [int ProductSubcategoryId]
        //@return [string]
        //**************************************************************************
        public string ProductSubCategory_Delete(int ProductSubcategoryId)
        {
            try
            {
                ProductsDataContext DataContextProduct = new ProductsDataContext();
                dtProductSubCategory_DeleteResult result = DataContextProduct.dtProductSubCategory_Delete(ProductSubcategoryId).FirstOrDefault();
                if (result.REGISTROS > 0) return "Categoría eliminada correctamente";
                else return "No ha sido posible eliminar la categoría, vuelva a intentarlo";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        //**************************************************************************
        //Nombre: GetProductLocation()
        //Descripción: Devuelve la localización de un producto en base al ID de la oferta.
        //@params [int ProductID]
        //@return [dtProductLocation_RecoveryByProductIdResult]
        //**************************************************************************
        public dtProductLocation_RecoveryByProductIdResult GetProductLocation(int ProductID)
        {
            ProductsDataContext DataContextProduct = new ProductsDataContext();
            return DataContextProduct.dtProductLocation_RecoveryByProductId(ProductID).FirstOrDefault();
        }

        //**************************************************************************
        //Nombre: getmaxProductBuy()
        //Descripción: Obtiene los valores del combo "Cantidad a comprar"
        //@params [Int32 IdProduct]
        //@return [List<int>]
        //**************************************************************************
        public List<int> getmaxProductBuy(Int32 IdProduct)
        {
            List<int> nummax = new List<int>();
            nummax.Add(1);
            nummax.Add(2);
            nummax.Add(3);
            nummax.Add(4);
            nummax.Add(5);
            nummax.Add(6);

            return nummax;
        }

        //**************************************************************************
        //Nombre: UpdateProductStatus()
        //Descripción: Actualiza el estado de un producto
        //@params [Int32 IdProduct, Int32 Status]
        //@return [string]
        //**************************************************************************
        public string UpdateProductStatus(int ProductID, int Status)
        {
            string result = "";
            try
            {
                ProductsDataContext DataContextProduct = new ProductsDataContext();
                DataContextProduct.dtProducts_UpdateProductStatus(ProductID, Status);
                result = "ok";
            }
            catch (Exception ex)
            {
                result = ex.Message.ToString();
            }

            return result;
        }

        //**************************************************************************
        //Nombre: MisCompras()
        //Descripción: Devuelve una lista de las compras realizadas por un usuario
        //@params [CustomerInfo CI]
        //@return [List<dtSalesOrder_RecoveryByUserResult>]
        //**************************************************************************
        public List<dtSalesOrder_RecoveryByUserResult> MisCompras(CustomerInfo CI)
        {
            try
            {
                // Creamos el contexto de datos    
                OrdersDataContext DataContextOrder = new OrdersDataContext();
                // Ejecutamos el stored para obtener la información de la base de datos
                List<dtSalesOrder_RecoveryByUserResult> SRU = DataContextOrder.dtSalesOrder_RecoveryByUser(CI.Customer.CustomerID).ToList();
                // Devolvemos los valores
                return SRU;
            }
            catch (Exception)
            {
                // Si hay algún error devolvemos null.
                return null;
            }
        }

        //**************************************************************************
        //Nombre: BuyOffer()
        //Descripción: Realiza el proceso de compra de la oferta.
        //@params [string NumProduct, string ProductID, string SelAddress, CustomerInfo CustomerInf]
        //@return [string]
        //**************************************************************************
        public string BuyOffer(string NumProduct, string ProductID, string SelAddress, CustomerInfo CustomerInf)
        {
            // Ejecutamos el proceso dentro del contexto de una transacción, si falla algo abortamos.
            TransactionOptions tso = new TransactionOptions();
            tso.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;

            using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required, tso))
            {
                try
                {
                    ProductModel ProductToBuy = getProductById(Convert.ToInt32(ProductID), CustomerInf.Customer.CustomerID);
                    decimal Subtotal = Convert.ToInt32(NumProduct) * ProductToBuy.OfferPrice;
                    OrdersDataContext OrderContextData = new OrdersDataContext();
                    string NumberOrder = DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + DateTime.Now.Hour.ToString() + DateTime.Now.Minute.ToString() + DateTime.Now.Second.ToString() + DateTime.Now.Millisecond.ToString();

                    int IDHeader = OrderContextData.dtSalesOrderHeader_InsertNew(1, 0, NumberOrder, CustomerInf.Customer.CustomerID, Convert.ToInt32(SelAddress),
                                                                  Convert.ToInt32(SelAddress), Subtotal, 21, 0, Subtotal, "");

                    int IDOrderDetail = OrderContextData.dtSalesOrderDetail_InsertNew(IDHeader, "0000", short.Parse(NumProduct), ProductToBuy.ProductID, ProductToBuy.Price, ProductToBuy.OfferPrice, Subtotal);
                    // Si todo ha ido bien, hacemos commit.
                    scope.Complete();
                    // Devolvemos el ID de la orden de compra.
                    return IDHeader.ToString();
                }
                catch (Exception ex)
                {
                    // Si algo falla hacemos rollback y devolvemos el error.
                    scope.Dispose();
                    return ex.ToString();
                }
                finally
                {
                    scope.Dispose();
                }

            }
        }
    }

}